"""
#    TOP2049 Open Source programming suite
#
#    Lowlevel USB hardware access.
#
#    Copyright (c) 2012 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from util import *
from command_queue import *
try:
	import usb.core
	import usb.util
except (ImportError), e:
	print("Python USB (PyUSB) support module not found.\n"
	      "Please install python-usb.")
	sys.exit(1)


class FoundUSBDev(object):
	def __init__(self, usbdev):
		self.usbdev = usbdev

class HardwareAccessUSB(CommandQueue):
	"Lowlevel USB hardware access"

	TIMEOUT_MSEC = 2000

	@classmethod
	def scan(cls, checkCallback):
		"Scan for devices. Returns a list of FoundUSBDev()."
		devices = list(usb.core.find(find_all = True,
					     custom_match = checkCallback))
		devices = [ FoundUSBDev(dev) for dev in devices ]
		return devices

	def __init__(self, usbdev, maxPacketBytes, noQueue,
		     doRawDump=False):
		CommandQueue.__init__(self,
				      maxPacketBytes = maxPacketBytes,
				      synchronous = noQueue)
		self.doRawDump = doRawDump
		self.usbdev = usbdev

		self.__initUSB()

	def __initUSB(self):
		try:
			config = self.usbdev.configurations()[0]
			interface = config.interfaces()[0]

			# Find the endpoints
			self.bulkOut = None
			self.bulkIn = None
			for ep in interface.endpoints():
				if not self.bulkIn and \
				   usb.util.endpoint_type(ep.bmAttributes) == usb.util.ENDPOINT_TYPE_BULK and \
				   usb.util.endpoint_direction(ep.bEndpointAddress) == usb.util.ENDPOINT_IN:
					self.bulkIn = ep
				if not self.bulkOut and \
				   usb.util.endpoint_type(ep.bmAttributes) == usb.util.ENDPOINT_TYPE_BULK and \
				   usb.util.endpoint_direction(ep.bEndpointAddress) == usb.util.ENDPOINT_OUT:
					self.bulkOut = ep
			if not self.bulkIn or not self.bulkOut:
				raise TOPException("Did not find all USB EPs")

			self.usbdev.set_configuration(config)
			self.bulkIn.clear_halt()
			self.bulkOut.clear_halt()

		except (usb.core.USBError), e:
			raise TOPException("USB error: " + str(e))

	def shutdown(self):
		"Shutdown the USB connection"
		try:
			usb.util.dispose_resources(self.usbdev)
		except (usb.core.USBError), e:
			raise TOPException("USB error: " + str(e))

	def send(self, data):
		try:
			assert(len(data) <= self.maxPacketBytes)
			if self.doRawDump:
				print("Sending command:")
				dumpMem(data)
			nrWritten = self.usbdev.write(
					self.bulkOut.bEndpointAddress,
					data,
					self.TIMEOUT_MSEC)
			if nrWritten != len(data):
				raise TOPException("USB bulk write error: "
					"short write")
		except (usb.core.USBError), e:
			raise TOPException("USB bulk write error: " + str(e))

	def receive(self, size):
		"""Receive 'size' bytes on the bulk-in ep."""
		# If there are blocked commands in the queue, send them now.
		self.flushCommands()
		try:
			ep = self.bulkIn.bEndpointAddress
			data = b"".join([ int2byte(b) for b in
					  self.usbdev.read(ep, size,
						self.TIMEOUT_MSEC) ])
			if len(data) != size:
				raise TOPException("USB bulk read error: Could not read the " +\
					"requested number of bytes (req %d, got %d)" % (size, len(data)))
			if self.doRawDump:
				print("Received data:")
				dumpMem(data)
		except (usb.core.USBError), e:
			raise TOPException("USB bulk read error: " + str(e))
		return data



"""
#    TOP2049 Open Source programming suite
#
#    Generic command queue.
#
#    Copyright (c) 2012 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from util import *
import time


class CommandQueue(object):
	"Generic hardware-command queue. Needs to be subclassed."

	def __init__(self, maxPacketBytes, synchronous=False):
		self.maxPacketBytes = maxPacketBytes
		self.synchronous = synchronous
		self.commandQueue = []

	def queueCommand(self, command):
		"""Queue a raw command for transmission."""
		assert(len(command) <= self.maxPacketBytes)
		if self.synchronous:
			self.send(command)
		else:
			self.commandQueue.append(command)

	def runCommandSync(self, command):
		"""Run a command synchronously.
		This is slow. Don't use it without a very good reason."""
		self.flushCommands()
		self.queueCommand(command)
		self.flushCommands()

	def flushCommands(self, sleepSeconds=0):
		"""Flush the command queue."""
		command = b""
		for oneCommand in self.commandQueue:
			assert(len(oneCommand) <= self.maxPacketBytes)
			if len(command) + len(oneCommand) > self.maxPacketBytes:
				self.send(command)
				command = b""
			command += oneCommand
		if command:
			self.send(command)
		self.commandQueue = []
		if sleepSeconds:
			time.sleep(sleepSeconds)

	def send(self, data):
		raise NotImplementedError # Reimplement in subclass.

	def receive(self, size):
		raise NotImplementedError # Reimplement in subclass.



"""
#    TOP2049 Open Source programming suite
#
#    Microchip8 common - basic file for 8bit PIC MCU
#
#    Copyright (c) 2012 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *
import math

class Chip_Microchip8_common(Chip):
	CMD_LOAD_DATA_FOR_PGM	 = 0x02
	CMD_LOAD_DATA_FOR_DM	 = 0x03
	CMD_READ_DATA_FROM_PGM	 = 0x04
	CMD_READ_DATA_FROM_DM	 = 0x05
	CMD_INCREMENT_ADDRESS	 = 0x06
	CMD_BULK_ERASE_PGM		 = 0x09
	CMD_BULK_ERASE_DM = 0x0B
	
	PCMDBIT_4BITINSTR 	 = 0
	PCMDBIT_SENDDATA	 = 1
	PCMDBIT_READDATA	 = 2
	
	STAT_BUSY = 0x01
	STAT_SDIO = 0x02

	# EEPROM access: default off, if exists override it
	hasEEPROM = False
	# Signature bytes access: default on, if doesn't exist, override it
	hasSigBytes = True

	# default delays - can be overridden
	delayTdly5 = 0.00000015
	delayTdis = 0.0001
	delayTprog = 0.001
	delayTdly = 0.000001
	delayTera = 0.01
	nLatches =1

	@classmethod
	def getSupportFlags(cls):
		flags = super(Chip_Microchip8_common, cls).getSupportFlags()
		if not cls.hasEEPROM:
			flags &= ~(Chip.SUPPORT_EEPROMREAD |\
				   Chip.SUPPORT_EEPROMWRITE)
		if not cls.hasSigBytes:
			flags &= ~Chip.SUPPORT_SIGREAD
		return flags

	def __init__(self,
			chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes
			):
		Chip.__init__(self,
		chipPackage=chipPackage,
		chipPinVCC=chipPinVCC,
		chipPinsVPP=chipPinsVPP,
		chipPinGND=chipPinGND)
		self.signature = signature
		self.flashPageSize = flashPageSize  # Flash page size, in words
		self.flashPages = flashPages  # Nr of flash pages
		self.eepromPageSize = eepromPageSize  # EEPROM page size, in bytes
		self.eepromPages = eepromPages  # Nr of EEPROM pages
		self.fuseBytes = fuseBytes  # Nr of fuse bytes
		self.PC = 0
		self.isInPmMode = False

	def erase(self):
		if(hasattr(self, 'osccalAddr')):
			self.__erase(keepOSCCAL=True)
		else:
			self.__erase(keepOSCCAL=False)
		
	def __erase(self, keepConfigWord=False, keepUserIDLocation=False, keepOSCCAL=False, keepEEPROM=False):
		OSCCAL = 0xfff
		self.exitPM()
		self.enterPM()
		if(keepOSCCAL):
			self.progressMeterInit("Reading OSCCAL)", 0)
			self.setPC(self.osccalAddr)
			self.sendReadFlashInstr()
			self.top.cmdDelay(self.delayTdly)
			self.readSDOBufferLow()
			self.readSDOBufferHigh()
			OSCCAL = self.top.cmdReadBufferReg16()
			self.progressMeterFinish()
			if(hasattr(self, 'osccalBackupAddr') and OSCCAL == 0xfff):
				self.progressMeterInit("OSCCAL value lost, restoring from backup location ...", 0)
				print("OSCCAL value lost, restoring from backup location ...")
				self.setPC(self.osccalBackupAddr - self.osccalAddr)
				self.sendReadFlashInstr()
				self.top.cmdDelay(self.delayTdly)
				self.readSDOBufferLow()
				self.readSDOBufferHigh()
				OSCCAL = self.top.cmdReadBufferReg16()
				self.progressMeterFinish()
				# print ("osccal: %x\n" % OSCCAL)
		if(keepConfigWord):
			self.progressMeterInit("Reading ConfigWord for backup", 0)
			CW = self.getConfigWord()
			self.progressMeterFinish()
		# erase User ID Location and backup osccal Tooo
		# erase User ID Location and backup osccal Tooo
		if(not keepUserIDLocation):
			self.enterConfigArea()
			self.setPC(self.userIDLocationAddr)
		self.progressMeterInit("Erasing chip", 0)
		self.bulkErasePGM()
		self.progressMeterFinish()
		#OSCCAL=0x3454
		#OSCCAL=0x0C10
		if(keepOSCCAL and OSCCAL != 0xfff):
			self.exitPM()
			self.enterPM()
			self.progressMeterInit("Writing osccal,  value %x" % OSCCAL, 0)
			print("Writing osccal,  value %x" % OSCCAL)
			self.setPC(self.osccalAddr)
			self.send6bitWriteInstruction(self.CMD_LOAD_DATA_FOR_PGM, OSCCAL)
			self.top.cmdDelay(self.delayTdly)
			self.sendWriteFlashInstr()
			self.progressMeterFinish()
		if(keepConfigWord):
			self.progressMeterInit("Writing ConfigWord, value %x" % CW, 0)
			self.writeConfigWord(CW)
			self.progressMeterFinish()
		if((not keepEEPROM) and self.hasEEPROM):
			self.progressMeterInit("Erasing EEPROM", 0)
			self.bulkEraseDM()
			self.progressMeterFinish()
		self.exitPM()

	def bulkErasePGM(self):
		self.sendCommand(0, 0, 0, self.CMD_BULK_ERASE_PGM)
		self.top.cmdDelay(self.delayTera)  # Tera
		
	def bulkEraseDM(self):
		self.sendCommand(0, 0, 0, self.CMD_BULK_ERASE_DM)
		self.top.cmdDelay(self.delayTera)  # Tera
		
	def readProgmem(self):	
		nrWords = self.flashPages * self.flashPageSize
		image = ""
		self.enterPM()
		self.setPC(0)
		self.progressMeterInit("Reading flash", nrWords)
		bufferedBytes = 0
		for word in range(0, nrWords):
			self.sendReadFlashInstr()
			# self.top.cmdDelay(0.00002) #20us wait - inconsistent data if skipped
			self.top.cmdDelay(self.delayTdly)
			
			self.readSDOBufferLow()
			bufferedBytes += 1
			self.readSDOBufferHigh()
			bufferedBytes += 1
			if bufferedBytes == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(bufferedBytes)
				self.progressMeter(word)
				bufferedBytes = 0
			self.incrementPC(1)
		image += self.top.cmdReadBufferReg(bufferedBytes)
		self.progressMeterFinish()
		# self.exitPM()
		return image
	
	def readEEPROM(self):	
		nrWords = self.eepromPages * self.eepromPageSize
		image = ""
		self.enterPM()
		self.progressMeterInit("Reading eeprom", nrWords)
		bufferedBytes = 0
		for word in range(0, nrWords):
			self.sendReadEEPROMInstr()
			self.top.cmdDelay(self.delayTdly)  # 20us wait - inconsistent data if skipped
			self.readSDOBufferLow()
			bufferedBytes += 1
			if bufferedBytes == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(bufferedBytes)
				self.progressMeter(word)
				bufferedBytes = 0
			self.incrementPC(1)
		image += self.top.cmdReadBufferReg(bufferedBytes)
		self.progressMeterFinish()
		# self.exitPM()
		return image

	def writeEEPROM(self, image):	
		nrWords = self.eepromPages * self.eepromPageSize
		if len(image) > nrWords:
			self.throwError("Invalid flash image size %d (expected <=%d)" % 	len(image))
		self.enterPM()
		self.progressMeterInit("Writing eeprom", nrWords)
		bufferedBytes = 0
		for addr in range(0, len(image)):
			self.progressMeter(addr)
			byte = byte2int(image[addr])
			if byte != 0xff:
				self.send6bitWriteInstruction(self.CMD_LOAD_DATA_FOR_DM, byte)
				self.top.cmdDelay(self.delayTdly)
				self.sendWriteFlashInstrDM()				
			self.incrementPC(1)
		self.progressMeterFinish()
		# self.exitPM()

	def writeProgmem(self, image):
		nrWords = self.flashPages * self.flashPageSize
		if len(image) > nrWords * 2 or len(image) % 2 != 0:
			self.throwError("Invalid flash image size %d (expected <=%d and word aligned)" % \
				(len(image), nrWords * 2))
		self.progressMeterInit("Writing flash", len(image) // 2)
		self.enterPM()
		self.setPC(0)
		latCnt=1;
		writeCurrentLatches=False
		for wordAddr in range(0, len(image) // 2):
			self.progressMeter(wordAddr)
			# do not swap following two lines
			WD = (byte2int(image[wordAddr * 2 + 1]) << 8) | byte2int(image[wordAddr * 2 + 0])
			if(WD != (byte2int(self.defaultWord[1]) << 8) + byte2int(self.defaultWord[0])):
				self.send6bitWriteInstruction(self.CMD_LOAD_DATA_FOR_PGM, WD)
				self.top.cmdDelay(self.delayTdly)				
				writeCurrentLatches=True
			if(latCnt == self.nLatches):
				if(writeCurrentLatches):
					self.sendWriteFlashInstr()
				latCnt=0
				writeCurrentLatches=False
			latCnt+=1
			self.incrementPC(1)
		if(latCnt>1):
			self.sendWriteFlashInstr()
		self.progressMeterFinish()
		# self.exitPM()

	def readFuse(self):
		self.enterPM()
		fuses = []
		self.progressMeterInit("Reading fuses (configuration word)", 0)
		for CW in self.getConfigWord(): 
			fuses.append(int2byte(CW & 0x00ff))
			fuses.append(int2byte((CW >> 8) & 0x00ff))
		self.progressMeterFinish()
		return b"".join(fuses)
	
	def readUserIdLocation(self):
		self.enterPM()
		self.enterConfigArea()
		self.setPC(self.userIDLocationAddr)
		self.progressMeterInit("Reading User ID Location", 0)
		for i in range(0, self.userIDLocationSize):
			self.sendReadFlashInstr()
			self.top.hostDelay(self.delayTdly)
			self.readSDOBufferLow()
			self.readSDOBufferHigh()
			self.incrementPC(1)
		# self.exitPM()
		self.progressMeterFinish()
		return self.top.cmdReadBufferReg()[0:2 * self.userIDLocationSize]
	
	def writeUserIdLocation(self, image):
		if len(image) > self.userIDLocationSize * 2 or len(image) % 2 != 0:
			self.throwError("Invalid flash image size %d (expected <=%d and word aligned)" % \
				(len(image), self.userIDLocationSize * 2))
		self.enterPM()
		self.enterConfigArea()
		self.setPC(self.userIDLocationAddr)
		self.progressMeterInit("Writing User ID Location", (len(image) // 2) - 1)
		for word in range(0, (len(image) // 2)):
			self.progressMeter(word)
			# do not swap following two lines
			WD = (byte2int(image[word * 2 + 1]) << 8) | byte2int(image[word * 2 + 0])
			if(WD != (byte2int(self.defaultWord[1]) << 8) + byte2int(self.defaultWord[0])):
				self.send6bitWriteInstruction(self.CMD_LOAD_DATA_FOR_PGM, WD)
				self.sendWriteFlashInstr()
			self.incrementPC(1)
		self.top.hostDelay(self.delayTdly)
		self.sendWriteFlashInstr()
		self.progressMeterFinish()
		# self.exitPM()
		
	def getConfigWordSize(self):
		return self.fuseBytes // 2

	def getConfigWord(self):
		self.enterPM()
		self.enterConfigArea()
		self.setPC(self.configWordAddr)
		retVal = []
		for i in range(0, self.getConfigWordSize()):
			self.sendReadFlashInstr()
			self.top.cmdDelay(self.delayTdly)
			self.readSDOBufferLow()
			self.readSDOBufferHigh()
			self.incrementPC(1)
			retVal.append(self.top.cmdReadBufferReg16())

		return retVal

	def writeFuse(self, image):
		if len(image) != 2 * self.getConfigWordSize():
			self.throwError("Invalid Fuses image size %d (expected %d)" % \
				(len(image), 2 * self.getConfigWordSize()))
		self.progressMeterInit("Writing fuses", 0)
		# print "image1:%x,,%x,,%x" % (byte2int(image[0]),byte2int(image[1]),byte2int(image[1])<<8)
		CW = []
		for tBytes in zip(image[::2], image[1::2]):
			CW.append((byte2int(tBytes[1]) << 8) | byte2int(tBytes[0]))
		self.writeConfigWord(CW)
		self.progressMeterFinish()
    
	def writeConfigWord(self, listConfigWord16):
		# Externally timed writes are not supported
		# for Configuration and Calibration bits. Any
		# externally timed write to the Configuration
		# or Calibration Word will have no effect on
		# the targeted word.
		self.enterPM()
		self.enterConfigArea()
		self.setPC(self.configWordAddr)
		for configWord16 in listConfigWord16:
			# print "write CW {:x}".format(configWord16)
			self.send6bitWriteInstruction(self.CMD_LOAD_DATA_FOR_PGM, configWord16)
			self.top.cmdDelay(self.delayTdly)
			self.sendWriteFlashInstrCW()
			self.incrementPC(1)
		self.top.flushCommands()
					

		
	def enterPM(self):
		if self.isInPmMode and self.isInsideProgramMemoryArea:
			self.resetPC()
			return
		self.PC = self.initPcValue
		self.isInsideProgramMemoryArea = True
		"Enter HV programming mode. Vdd first entry mode"
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(False)
		self.setPins(0, 0)
		self.top.cmdSetVCCVoltage(self.voltageVDD)
		self.top.cmdSetVPPVoltage(self.voltageVPP)
		# self.top.cmdEnableZifPullups(True)
		self.applyGND(True)
		
		self.applyVPP(True)
		self.top.cmdDelay(0.000250)
		
		
		self.applyVCC(True)
		# self.top.cmdEnableZifPullups(True)
		
		# self.top.cmdDelay(0.000005) #least 5us is required to reach Vdd first entry PM
		self.setTopProgrammerDelays()
		self.isInPmMode = True

	def checkSignature(self):
		signature = self.readSignature()
		if signature != self.signature:
			msg = "Unexpected device signature. " + \
			 "Want %02X%02X%02X, but got %02X%02X%02X" % \
			(byte2int(self.signature[0]), byte2int(self.signature[1]),
			byte2int(self.signature[2]),
			byte2int(signature[0]), byte2int(signature[1]),
			byte2int(signature[2]))
		if self.top.getForceLevel() >= 1:
			self.printWarning(msg)
		else:
			self.throwError(msg)

	def exitPM(self):
		"Exit HV programming mode. Vdd last exit mode"
		self.top.flushCommands()
		self.setPins(0, 0)
		self.applyVPP(False)
		self.applyGND(False)
		self.top.hostDelay(self.delayTdly)
		self.applyVCC(False)
		self.isInPmMode = False

	def sendReadFlashInstr(self):
		'''
		'''
		self.sendCommand(0, 0, 1, self.CMD_READ_DATA_FROM_PGM)

	def sendWriteFlashInstr(self):
		'''
		to be overriden
		'''
		pass
		
	def sendReadEEPROMInstr(self):
		'''
		'''
		self.sendCommand(0, 0, 1, self.CMD_READ_DATA_FROM_DM)

	def send6bitReadInstruction(self, pInstruction):
		def incBbAndCheckFillImage():
			self.BufferedBytes += 1
			if self.BufferedBytes == self.top.getBufferRegSize():
				self.flushBufferToImage()
		# self.sendCommand(1,0,1,pInstruction)
		self.sendCommand(0, 0, 1, pInstruction)
		# self.busyWait()
		self.readSDOBufferHigh()
		incBbAndCheckFillImage()
		
	def send6bitWriteInstruction(self, pInstruction, pDataPayload):
		# self.busyWait()
		self.setSDI(pDataPayload)
		# print("sending {:x}\n".format(pDataPayload))
		self.sendCommand(0, 1, 0, pInstruction)
		self.top.flushCommands()

	def setSDI(self, sdi):
		'''
		set 14 bit sdi value
		'''
		self.top.cmdFPGAWrite(0x13, sdi & 0xFF)
		self.top.cmdFPGAWrite(0x14, (sdi >> 8) & 0x3F)

	def sendCommand(self, bit4bitInstr=0, bitSendData=0, bitReadData=0, cmd4bit=0):
		'''
		`define CMDBIT_4BITINSTR 0
		`define CMDBIT_SENDDATA	1
		`define CMDBIT_READDATA	2
		'''
		command = (cmd4bit & 0x1F) << 3
		if bit4bitInstr:
			command |= 2 ** self.PCMDBIT_4BITINSTR
		if bitSendData:
			command |= 2 ** self.PCMDBIT_SENDDATA
		if bitReadData:
			command |= 2 ** self.PCMDBIT_READDATA
		# print("cmd sending {:x}\n".format(command))
		self.top.cmdFPGAWrite(0x12, command)
		
	def setTopProgrammerDelays(self):
		#print("tdel5:{:d}".format(int(math.ceil(self.delayTdly5 / 42e-9))))
		#print("tdly:{:d}".format(int(math.ceil(self.delayTdly / 42e-9))))
		self.top.cmdFPGAWrite(0x10, int(math.ceil(self.delayTdly5 / 42e-9)))
		self.top.cmdFPGAWrite(0x11, int(math.ceil(self.delayTdly / 42e-9)))

# 	def runCommandSync(self, command):
# 		self.loadCommand(command)
# 		self.busyWait()
	def resetPC(self):
		'''can be overriden'''
		self.setPC(self.initPcValue)
			
	def setPC(self, address):
		'''to be overriden'''
		pass

	def incrementPC(self, count):
		'''to be overriden'''
		pass
	
	def enterConfigArea(self, wordLatched=0):
		'''to be overriden'''
		pass

	def setPins(self, ICSPCLK=0, SDIOVALUE=0, SDIODRIVEN=1):
		'''
		setPins
		'''
		data = 0
		if ICSPCLK:
			data |= 1
		if SDIODRIVEN:
			data |= 2
		if SDIOVALUE:
			data |= 4	    
		self.top.cmdFPGAWrite(0x15, data)

	def getStatusFlags(self):
		'''
		'''
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def readSDOBufferHigh(self):
		self.top.cmdFPGARead(0x10)
	
	def readSDOBufferLow(self):
		self.top.cmdFPGARead(0x13)	
	
	def rawSDIOState(self):
		return bool(self.getStatusFlags() & self.STAT_SDIO)

	def busy(self):
		return bool(self.getStatusFlags() & self.STAT_BUSY)

# 	def busyWait(self):
# 		for i in range(0, 100):
# 			if not self.busy():
# 				return
# 			self.top.hostDelay(0.01)
# 		self.throwError("Timeout in busywait.")

# 	def waitHighSDIO(self):
# 		for i in range(0, 100):
# 			if self.rawSDOState():
# 				return
# 			self.top.hostDelay(0.01)
# 		self.throwError("Timeout waiting for SDO.")



"""
#    TOP2049 Open Source programming suite
#
#    Microchip8_18_common - basic file for 8bit PIC18 MCU
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *

class Chip_Microchip8_18_common(Chip):
	
	STAT_BUSY = 0x01
	STAT_SDIO = 0x02

	
	PCMDBIT_4BITINSTR 	 = 0
	PCMDBIT_SENDDATA		 = 1
	PCMDBIT_READDATA		 = 2	
	PCMDBIT_KEEPCLKHIGH	 = 7	
	
	CMD_CORE_INSTRUCTION	 = 0x0
	CMD_SHIFT_OUT_TABLAT	 = 0x2
	CMD_TR					 = 0x8
	CMD_TRI					 = 0x9
	CMD_TRD					 = 0xA
	CMD_ITR					 = 0xB
	CMD_TW					 = 0xC
	CMD_TWII				 = 0xD
	CMD_TWDD				 = 0xE
	CMD_TW_START_PROG		 = 0xF

	# EEPROM access: default on, if does not exist override it
	hasEEPROM = True

	# default delays - can be overridden
	delayP2A = 400e-9  # Serial clock low time 
  	delayP5 = 2.2e-6  # Delay between 4-bit command and command operand
  	delayP5A = 2.2e-6  # Delay between 4-bit command operand and next 4-bit command 
  	delayP6 = 2.2e-6  # Delay between last SCK fall of command byte to first SCK rise of read data word
  	delayP9 = 1e-3  # SCK High time (minimum programming time)
  	delayP10 = 30e-6  # SCK Low time after programming (high-voltage discharge time)
  	delayP11 = 0.01  # Delay to allow self-timed data write or bulk erase to occur
  	delayP12 = 0.000002  # Input data hold time from nMCLR/Vpp rise
  	delayP13 = 0.0000001  # Vdd rise setup time to nMCLR/Vpp rise
  	delayP14 = 0.00000001  # Data out Valid from SCK rise
  	delayP15 = 0.000002  # PGM rise setup time to nMCLR/Vpp rise
  	
	userIDLocationSize		 = 8
	userIDLocationAddr  	 = 0x200000

	deviceIDAddr 			 = 0x3FFFFE
	configWordAddr			 = 0x300000
	deviceIDLength 			 = 2
	voltageVDD				 = 5
	voltageVPP				 = 12

	def __init__(self,
			chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes
			):
		Chip.__init__(self,
		chipPackage=chipPackage,
		chipPinVCC=chipPinVCC,
		chipPinsVPP=chipPinsVPP,
		chipPinGND=chipPinGND)
		self.signature = signature
		self.flashPageSize = flashPageSize  # Flash page size, in words
		self.flashPages = flashPages  # Nr of flash pages
		self.eepromPageSize = eepromPageSize  # EEPROM page size, in bytes
		self.eepromPages = eepromPages  # Nr of EEPROM pages
		self.fuseBytes = fuseBytes  # Nr of fuse bytes
		self.isInPmMode = False
		self.BufferedBytes = 0
		self.Image = ""

	def getIHexInterpreter(self):
		inter = IHexInterpreter()
		inter.progmemRanges = [ AddressRange(0, self.flashPageSize) ]
		inter.fuseRanges = [ AddressRange(self.configWordAddr,
						  self.configWordAddr + self.fuseBytes) ]
		inter.uilRanges = [ AddressRange(self.userIDLocationAddr,
						 self.userIDLocationAddr + self.userIDLocationSize) ]
		return inter

	def enterPM(self, force=False):
		if self.isInPmMode and not force:
			return
		"Enter HV programming mode. Vdd first entry mode"
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(False)
		self.setPins(0, 0)
		self.top.cmdSetVCCVoltage(self.voltageVDD)
		self.top.cmdSetVPPVoltage(self.voltageVPP)
		self.applyGND(True)
		self.applyVCC(True)
		self.top.hostDelay(10 * self.delayP13)
		self.applyVPP(True)
		self.top.hostDelay(102 * self.delayP12)
		self.setTopProgrammerDelays()
		self.isInPmMode = True
	
	def readUserIdLocation(self):
		return self.readSequentialBlock(self.userIDLocationAddr, self.userIDLocationSize, "Reading User ID Locations")
	
	def readFuse(self):
		return self.readSequentialBlock(self.configWordAddr, self.fuseBytes, "Reading Config Words")
	
	def readSignature(self):
		return self.readSequentialBlock(self.deviceIDAddr, self.deviceIDLength, "Reading Signature")

	def readProgmem(self):	
		nrBytes = self.flashPages * self.flashPageSize
		return self.readSequentialBlock(0, nrBytes, "Reading flash")
	
	def readSequentialBlock(self, startAddr, nBytes, infoText):	
		self.enterPM()
		self.progressMeterInit(infoText, nBytes)
		self.BufferedBytes = 0
		self.Image = ""
		self.executeCode(self.getCodeAddrToTBLPTR(startAddr))
		for byteAddr in range(0, nBytes):
			self.send4bitReadInstruction(self.CMD_TRI)
			self.progressMeter(byteAddr)
		self.progressMeterFinish()
		self.flushBufferToImage()
		return self.Image
	
	def writeSequentialBlock(self, startAddr, image, size, infoText):
		if len(image) > size:
			self.throwError("Invalid flash image size %d (expected <=%d)" % \
				(len(image), self.userIDLocationSize))
		self.enterPM()
		self.executeCode((0x8EA6, 0x9CA6))
		self.progressMeterInit(infoText, len(image) / 8)
		for blockAddr in range(0, len(image), self.writeBufferSize):
			#print("addr:{:x}".format(startAddr+blockAddr))
			self.executeCode(self.getCodeAddrToTBLPTR(startAddr+blockAddr))
			#for code in self.getCodeAddrToTBLPTR(startAddr+blockAddr):
			#	print("({:x}, ".format(code))
			print(")\n")
			self.writeNbytes(image[blockAddr:], self.writeBufferSize)
			#self.executeCode((0x0, 0x0))
			self.progressMeter(blockAddr)
		self.progressMeterFinish()
			
	def readEEPROM(self):      
		nrBytes = self.eepromPages * self.eepromPageSize
		self.enterPM()
		self.progressMeterInit("Reading EEPROM", nrBytes)
		self.BufferedBytes = 0
		self.Image = ""
		self.executeCode((0x9EA6, 0x9CA6))
		for byteAddr in range(0, nrBytes):
			# print("set addr to {:x}\n".format(byteAddr))
			self.setEEPROMAddr(byteAddr)
			self.executeCode((0x80A6, 0x50A8, 0x6EF5))
			self.send4bitReadInstruction(self.CMD_SHIFT_OUT_TABLAT)
			self.progressMeter(byteAddr)
		self.progressMeterFinish()
		self.flushBufferToImage()
		return self.Image
	
	def writeEEPROM(self, image):	
		nrBytes = self.eepromPages * self.eepromPageSize
		if len(image) > nrBytes:
			self.throwError("Invalid flash image size {:d} (expected <={:d})".format(len(image), nrBytes))
		self.enterPM()
		self.progressMeterInit("Writing eeprom", len(image))
		self.executeCode((0x9EA6, 0x9CA6))
		for addr in range(0, len(image)):
			self.progressMeter(addr)
			#print("writing {:x} value to addr {:x}\n".format(byte2int(image[addr]), addr))
			self.setEEPROMAddr(addr)
			self.executeCode((0x0E00 | (byte2int(image[addr]) & 0xFF), 0x6EA8))
			self.executeCode((0x84A6, 0x0E55, 0x6EA7, 0x0EAA, 0x6EA7))
			self.executeCode((0x82A6, 0x0, 0x0))
			self.top.hostDelay(self.delayP11 + self.delayP10)
			self.executeCode((0x94A6,))
		self.progressMeterFinish()

	def writeNbytes(self, image, N):
		if N % 2:
			self.throwError("N should be even, not %d" % N)	
		isEmpty = True
		#N = (pN, len(image))[len(image) < pN]
		for idx in range(0, N):
			if idx == len(image):
				image += '\xFF'
			elif image[idx] != '\xFF':
				isEmpty = False
		if(not isEmpty):
			for wordAddr in range(0, N-2, 2):
				self.send4bitWriteInstruction(self.CMD_TWII, byte2int(image[wordAddr]) | (byte2int(image[wordAddr + 1]) << 8))
			self.send4bitWriteInstruction(self.CMD_TW_START_PROG, byte2int(image[N-2]) | (byte2int(image[N-1]) << 8))
			self.top.cmdFPGAWrite(0x12, 0x81)
			self.top.hostDelay(self.delayP9)
			self.setPins(0)
			self.top.cmdDelay(self.delayP10)
			for i in range(0,4):
				self.sendCommand(1)

	def writeUserIdLocation(self, image):
		self.writeSequentialBlock(self.userIDLocationAddr, image, self.userIDLocationSize, "Writing User ID Locations")

	def checkSignature(self):
		signature = self.readSignature()
		if signature != self.signature:
			msg = "Unexpected device signature. " + \
			 "Want %02X%02X%02X, but got %02X%02X%02X" % \
			(byte2int(self.signature[0]), byte2int(self.signature[1]),
			byte2int(self.signature[2]),
			byte2int(signature[0]), byte2int(signature[1]),
			byte2int(signature[2]))
		if self.top.getForceLevel() >= 1:
			self.printWarning(msg)
		else:
			self.throwError(msg)

	def writeProgmem(self, image):
		nrBytes = self.flashPages * self.flashPageSize
		if len(image) > nrBytes:
			self.throwError("Invalid flash image size %d (expected <=%d)" % \
				(len(image), nrBytes))
		self.writeSequentialBlock(0, image, nrBytes, "Writing flash")

	def writeFuse(self, image):
		self.enterPM()
		if len(image) > self.fuseBytes:
			self.throwError("Invalid Fuses image size %d (expected less than %d)" % \
				(len(image), self.fuseBytes))		
		self.executeCode((0x8EA6, 0x8CA6, 0xEF00, 0xF800))
		for fuseAddr in range(0,len(image)):
			self.executeCode(self.getCodeAddrToTBLPTR(self.configWordAddr+fuseAddr))
			if(fuseAddr & 0x01):
				byte =  byte2int(image[fuseAddr]) << 8
			else:
				byte =  byte2int(image[fuseAddr])
			self.send4bitWriteInstruction(self.CMD_TW_START_PROG, byte)
			self.top.cmdFPGAWrite(0x12, 0x81)
			#self.setPins(1)
			self.top.hostDelay(self.delayP9)
			self.setPins(0)
			self.top.cmdDelay(self.delayP10)
			for i in range(0,4):
				self.sendCommand(1)
			#self.executeCode((0x2AF6,))
		self.writeSequentialBlock(self.configWordAddr, image, self.fuseBytes, "Writing fuses")
		self.progressMeterInit("Writing fuses", 0)
						
	def exitPM(self):
		"Exit programming mode. Vdd last exit mode"
		self.top.flushCommands()
		self.setPins(0, 0)
		self.applyVPP(False)
		self.applyVCC(False)
		self.applyGND(False)
		self.isInPmMode = False

# ready for 18F below
	def send4bitReadInstruction(self, pInstruction):
		def incBbAndCheckFillImage():
			self.BufferedBytes += 1
			if self.BufferedBytes == self.top.getBufferRegSize():
				self.flushBufferToImage()
		# self.sendCommand(1,0,1,pInstruction)
		self.sendCommand(1, 0, 1, pInstruction)
		# self.busyWait()
		self.readSDOBufferHigh()
		incBbAndCheckFillImage()
		
	def send4bitWriteInstruction(self, pInstruction, pDataPayload):
		# self.busyWait()
		self.setSDI(pDataPayload)
		#print("sending {:x}\n".format(pDataPayload))
		self.sendCommand(1, 1, 0, pInstruction)
		self.top.flushCommands()
		
	def sendCommand(self, bit4bitInstr=1, bitSendData=0, bitReadData=0, cmd4bit=0, bitKeepClkHigh=0):
		'''
		`define CMDBIT_4BITINSTR 0
		`define CMDBIT_SENDDATA	1
		`define CMDBIT_READDATA	2
		`define CMDBIT_KEEPCLKHIGH 7
		'''
		command = (cmd4bit & 0x0F) << 3
		if bit4bitInstr:
			command |= 2 ** self.PCMDBIT_4BITINSTR
		if bitSendData:
			command |= 2 ** self.PCMDBIT_SENDDATA
		if bitReadData:
			command |= 2 ** self.PCMDBIT_READDATA
		if bitKeepClkHigh:
			command |= 2 ** self.PCMDBIT_KEEPCLKHIGH
		# print("cmd sending {:x}\n".format(command))
		self.top.cmdFPGAWrite(0x12, command)
		if(bitSendData or bitReadData):
			self.top.cmdDelay(2 * 20 * 2 * self.delayP2A)
		else:
			self.top.cmdDelay(2 * 4 * 2 * self.delayP2A)
		
	def setTopProgrammerDelays(self):
		#print("tdel5:{:d}".format(int(math.ceil(self.delayP2A / 42e-9))))
		#print("tdly:{:d}".format(int(math.ceil(self.delayP5 / 42e-9))))
		self.top.cmdFPGAWrite(0x10, int(math.ceil(self.delayP2A / 42e-9)))
		self.top.cmdFPGAWrite(0x11, int(math.ceil(self.delayP5 / 42e-9)))		

	def setSDI8(self, sdi):
		self.top.cmdFPGAWrite(0x16, sdi & 0xFF)
			
	def setSDI(self, sdi):
		'''
		16 -set 16 bit sdi value
		'''
		for addr in (0x16, 0x17):
			self.top.cmdFPGAWrite(addr, sdi & 0xFF)
			sdi = sdi >> 8
		
	def flushBufferToImage(self):
		# print ("storing {:d} bytes to image".format(self.BufferedBytes))
		if self.BufferedBytes > 0:
			self.Image += self.top.cmdReadBufferReg(self.BufferedBytes)
			self.BufferedBytes = 0
	
	def sendInstruction(self, instr):
		self.setSDI(instr)
		self.sendCommand(1, 1)  # send 4 times positive edge
		# self.top.flushCommands()
		
	def executeCode(self, code):
		for instr in code:
			self.sendInstruction(instr)

	def setPins(self, ICSPCLK=0, SDIOVALUE=0, SDIODRIVEN=1):
		'''
		16 - setPins
		'''
		data = 0
		if ICSPCLK:
			data |= 1
		if SDIODRIVEN:
			data |= 2
		if SDIOVALUE:
			data |= 4	    
		self.top.cmdFPGAWrite(0x15, data)

	def getStatusFlags(self):
		'''
		[0] - BUSY
		[1] - SDO
		'''
		self.flushBufferToImage()
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def readSDOBufferHigh(self):
		self.top.cmdFPGARead(0x14)
	
	def readSDOBufferLow(self):
		self.top.cmdFPGARead(0x15)	
	
	def rawSDIOState(self):
		return bool(self.getStatusFlags() & self.STAT_SDIO)
	
	def isBusy(self):
		return bool(self.getStatusFlags() & self.STAT_BUSY)

	def busyWait(self):
		for i in range(0, 100):
			if not self.isBusy():
				return
		       	self.top.hostDelay(0.000001)
		self.throwError("Timeout in busywait.")

	def getCodeAddrToTBLPTR(self, addr):
		ub = (addr >> 16) & 0xFF
		hb = (addr >> 8) & 0xFF
		lb = addr & 0xFF
		return ((0x0E00 | ub), 0x6EF8, (0x0E00 | hb), 0x6EF7, (0x0E00 | lb), 0x6EF6)



"""
#    TOP2049 Open Source programming suite
#
#    Implements the Atmel Mega MCU parallel HV programming algorithm
#
#    Copyright (c) 2009-2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_ATMega_common(Chip):
	# The Atmel Mega programming commands
	CMD_CHIPERASE		= 0x80 # Chip Erase
	CMD_WRITEFUSE		= 0x40 # Write Fuse Bits
	CMD_WRITELOCK		= 0x20 # Write Lock Bits
	CMD_WRITEFLASH		= 0x10 # Write Flash
	CMD_WRITEEEPROM		= 0x11 # Write EEPROM
	CMD_READSIG		= 0x08 # Read Signature bytes and Calibration byte
	CMD_READFUSELOCK	= 0x04 # Read Fuse and Lock bits
	CMD_READFLASH		= 0x02 # Read Flash
	CMD_READEEPROM		= 0x03 # Read EEPROM

	def __init__(self,
		     chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
		     signature,
		     flashPageSize, flashPages,
		     eepromPageSize, eepromPages,
		     fuseBytes
		    ):
		Chip.__init__(self,
			      chipPackage = chipPackage,
			      chipPinVCC = chipPinVCC,
			      chipPinsVPP = chipPinsVPP,
			      chipPinGND = chipPinGND)
		self.signature = signature
		self.flashPageSize = flashPageSize	# Flash page size, in words
		self.flashPages = flashPages		# Nr of flash pages
		self.eepromPageSize = eepromPageSize	# EEPROM page size, in bytes
		self.eepromPages = eepromPages		# Nr of EEPROM pages
		self.fuseBytes = fuseBytes		# Nr of fuse bytes

	def readSignature(self):
		self.__enterPM()

		(signature, calibration) = self.__readSigAndCalib()

		return signature

	def erase(self):
		self.__enterPM()

		self.progressMeterInit("Erasing chip", 0)
		self.__loadCommand(self.CMD_CHIPERASE)
		self.__pulseWR()
		self.__waitForRDY()
		self.progressMeterFinish()

	def readProgmem(self):
		self.__enterPM()

		self.progressMeterInit("Reading Flash", self.flashPages)
		image = ""
		for page in range(0, self.flashPages):
			self.progressMeter(page)
			readWords = 0
			for word in range(0, self.flashPageSize):
				self.__loadCommand(self.CMD_READFLASH)
				self.__loadAddr((page * self.flashPageSize) + word)
				self.__readWordToStatusReg()
				readWords += 1
				if readWords >= 32:
					image += self.top.cmdReadBufferReg()
					readWords = 0
			if readWords:
				data = self.top.cmdReadBufferReg()
				image += data[0:readWords*2]
		self.progressMeterFinish()
		return image

	def writeProgmem(self, image):
		flashBytes = self.flashPageSize * 2 * self.flashPages
		if len(image) != flashBytes:
			self.throwError("Invalid program memory image size %d (expected %d)" %\
				(len(image), flashBytes))
		self.__enterPM()

		self.progressMeterInit("Writing Flash", self.flashPages)
		for page in range(0, self.flashPages):
			self.progressMeter(page)
			for word in range(0, self.flashPageSize):
				self.__loadCommand(self.CMD_WRITEFLASH)
				addr = (page * self.flashPageSize) + word
				self.__loadAddr(addr)
				addr *= 2
				data = image[addr : addr + 2]
				self.__loadData(byte2int(data[0]) | (byte2int(data[1]) << 8))
				self.__setBS1(1)
				self.__pulsePAGEL()
			self.__setBS1(0)
			self.__pulseWR()
			self.__waitForRDY()
		self.progressMeterFinish()

	def readEEPROM(self):
		self.__enterPM()

		assert(self.eepromPageSize <= self.top.getBufferRegSize())
		self.progressMeterInit("Reading EEPROM", self.eepromPages)
		image = ""
		for page in range(0, self.eepromPages):
			self.progressMeter(page)
			for byte in range(0, self.eepromPageSize):
				self.__loadCommand(self.CMD_READEEPROM)
				self.__loadAddr((page * self.eepromPageSize) + byte)
				self.__readLowByteToStatusReg()
			data = self.top.cmdReadBufferReg()
			image += data[0:self.eepromPageSize]
		self.progressMeterFinish()
		return image

	def writeEEPROM(self, image):
		eepromBytes = self.eepromPageSize * self.eepromPages
		if len(image) != eepromBytes:
			self.throwError("Invalid EEPROM image size %d (expected %d)" %\
				(len(image), eepromBytes))
		self.__enterPM()

		self.progressMeterInit("Writing EEPROM", self.eepromPages)
		for page in range(0, self.eepromPages):
			self.progressMeter(page)
			for byte in range(0, self.eepromPageSize):
				self.__loadCommand(self.CMD_WRITEEEPROM)
				addr = (page * self.eepromPageSize) + byte
				self.__loadAddr(addr)
				data = image[addr]
				self.__loadDataLow(byte2int(data[0]))
				self.__pulsePAGEL()
			self.__setBS1(0)
			self.__pulseWR()
			self.__waitForRDY()
		self.progressMeterFinish()

	def readFuse(self):
		self.__enterPM()

		self.progressMeterInit("Reading Fuse bits", 0)
		(fuse, lock) = self.__readFuseAndLockBits()
		self.progressMeterFinish()
		return fuse

	def writeFuse(self, image):
		if len(image) != self.fuseBytes:
			self.throwError("Invalid Fuses image size %d (expected %d)" %\
				(len(image), self.fuseBytes))
		self.__enterPM()

		self.progressMeterInit("Writing Fuse bits", 0)

		self.__loadCommand(self.CMD_WRITEFUSE)
		self.__setBS2(0)
		self.__loadDataLow(byte2int(image[0]))
		self.__setBS1(0)
		self.__pulseWR()
		self.__waitForRDY()

		self.__loadCommand(self.CMD_WRITEFUSE)
		self.__setBS2(0)
		self.__loadDataLow(byte2int(image[1]))
		self.__setBS1(1)
		self.__pulseWR()
		self.__waitForRDY()

		if len(image) >= 3:
			self.__loadCommand(self.CMD_WRITEFUSE)
			self.__setBS2(1)
			self.__loadDataLow(byte2int(image[2]))
			self.__setBS1(0)
			self.__pulseWR()
			self.__waitForRDY()

		self.__setBS1(0)
		self.__setBS2(0)

		self.progressMeterFinish()

	def readLockbits(self):
		self.__enterPM()

		self.progressMeterInit("Reading lock bits", 0)
		(fuses, lockbits) = self.__readFuseAndLockBits()
		self.progressMeterFinish()

		return lockbits

	def writeLockbits(self, image):
		if len(image) != 1:
			self.throwError("Invalid lock-bits image size %d (expected %d)" %\
				(len(image), 1))
		self.__enterPM()

		self.progressMeterInit("Writing lock bits", 0)
		self.__loadCommand(self.CMD_WRITELOCK)
		self.__loadDataLow(byte2int(image[0]))
		self.__pulseWR()
		self.__waitForRDY()
		self.progressMeterFinish()

	def __readSigAndCalib(self):
		"""Reads the signature and calibration bytes and returns them.
		This function expects a DUT present and pins initialized."""
		signature = ""
		calibration = ""
		for addr in range(0, 3):
			self.__loadCommand(self.CMD_READSIG)
			self.__loadAddr(addr)
			self.__readWordToStatusReg()
			data = self.top.cmdReadBufferReg()
			if addr == 0:
				calibration += data[1]
			signature += data[0]
		return (signature, calibration)

	def __readFuseAndLockBits(self):
		"""Reads the Fuse and Lock bits and returns them.
		This function expects a DUT present and pins initialized."""
		self.__loadCommand(self.CMD_READFUSELOCK)
		self.__setBS2(0)
		self.__readWordToStatusReg()
		self.__setBS2(1)
		self.__readWordToStatusReg()
		self.__setBS2(0)
		data = self.top.cmdReadBufferReg()
		if self.fuseBytes == 2:
			# fuseLow, fuseHigh
			fuses = data[0] + data[3]
		elif self.fuseBytes == 3:
			# fuseLow, fuseHigh, fuseExt
			fuses = data[0] + data[3] + data[2]
		else:
			assert(0)
		lock = data[1]
		return (fuses, lock)

	def __enterPM(self):
		"Enter HV programming mode."
		self.applyVPP(False)
		self.applyVCC(False)
		self.applyGND(True)
		self.top.cmdSetVPPVoltage(0)
		self.top.cmdSetVPPVoltage(12)
		self.top.cmdSetVCCVoltage(5)

		self.__setVoltageControl(VPP_en=1, VPP=0, VCC_en=1, VCC=0)
		self.__setXA0(0)
		self.__setXA1(0)
		self.__setBS1(0)
		self.__setPAGEL(0)
		self.__setWR(0)
		self.top.hostDelay(0.1)

		self.applyVCC(True)
		self.__setVoltageControl(VPP_en=1, VPP=0, VCC_en=1, VCC=1)
		self.top.hostDelay(0.1)

		self.__setOE(0)
		self.__setWR(1)
		self.__setXTAL1(0)
		self.__setXA0(0)
		self.__setXA1(0)
		self.__setBS1(0)
		self.__setBS2(0)
		self.__setPAGEL(0)
		self.__pulseXTAL1(10)
		self.top.flushCommands()

		self.__setVoltageControl(VPP_en=0, VPP=0, VCC_en=1, VCC=1)
		self.applyVPP(True)

		self.__setOE(1)

		(signature, calibration) = self.__readSigAndCalib()
		if signature != self.signature:
			msg = "Unexpected device signature. " +\
			      "Want %02X%02X%02X, but got %02X%02X%02X" % \
				(byte2int(self.signature[0]), byte2int(self.signature[1]),
				 byte2int(self.signature[2]),
				 byte2int(signature[0]), byte2int(signature[1]),
				 byte2int(signature[2]))
			if self.top.getForceLevel() >= 1:
				self.printWarning(msg)
			else:
				self.throwError(msg)

	def __readWordToStatusReg(self):
		"""Read a data word from the DUT into the status register."""
		self.__setBS1(0)
		self.__setOE(0)
		self.top.cmdFPGARead(0x10)
		self.__setBS1(1)
		self.top.cmdFPGARead(0x10)
		self.__setOE(1)

	def __readLowByteToStatusReg(self):
		"""Read the low data byte from the DUT into the status register."""
		self.__setBS1(0)
		self.__setOE(0)
		self.top.cmdFPGARead(0x10)
		self.__setOE(1)

	def __readHighByteToStatusReg(self):
		"""Read the high data byte from the DUT into the status register."""
		self.__setBS1(1)
		self.__setOE(0)
		self.top.cmdFPGARead(0x10)
		self.__setOE(1)

	def __loadData(self, data):
		"""Load a data word."""
		self.__loadDataLow(data)
		self.__loadDataHigh(data >> 8)

	def __loadDataLow(self, dataLow):
		"""Load the low data byte."""
		self.__setBS1(0)
		self.__setXA0(1)
		self.__setXA1(0)
		self.top.cmdFPGAWrite(0x10, dataLow & 0xFF)
		self.__pulseXTAL1()

	def __loadDataHigh(self, dataHigh):
		"""Load the high data byte."""
		self.__setBS1(1)
		self.__setXA0(1)
		self.__setXA1(0)
		self.top.cmdFPGAWrite(0x10, dataHigh & 0xFF)
		self.__pulseXTAL1()

	def __loadAddr(self, addr):
		"""Load an address word."""
		self.__loadAddrLow(addr)
		self.__loadAddrHigh(addr >> 8)

	def __loadAddrLow(self, addrLow):
		"""Load the low address byte."""
		self.__setBS1(0)
		self.__setXA0(0)
		self.__setXA1(0)
		self.top.cmdFPGAWrite(0x10, addrLow & 0xFF)
		self.__pulseXTAL1()

	def __loadAddrHigh(self, addrHigh):
		"""Load the high address byte."""
		self.__setBS1(1)
		self.__setXA0(0)
		self.__setXA1(0)
		self.top.cmdFPGAWrite(0x10, addrHigh & 0xFF)
		self.__pulseXTAL1()

	def __loadCommand(self, command):
		"""Load a command into the device."""
#		self.top.queueCommand("\x34")
		self.__setBS1(0)
#		self.top.queueCommand("\x34")
		self.__setXA0(0)
		self.__setXA1(1)
		self.top.cmdFPGAWrite(0x10, command)
		self.__pulseXTAL1()

	def __waitForRDY(self):
		"""Wait for the RDY pin to go high."""
		self.top.hostDelay(0.01)
		for i in range(0, 50):
			if self.__getRDY():
				return
			self.top.hostDelay(0.01)
		self.throwError("Timeout waiting for READY signal from chip.")

	def __getRDY(self):
		"""Read the state of the RDY/BSY pin."""
		return bool(self.__getStatus() & 0x01)

	def __getStatus(self):
		"""Read the programmer status register"""
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def __setOE(self, high):
		"""Set the OE pin of the DUT"""
		value = 0x02
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __setWR(self, high):
		"""Set the WR pin of the DUT"""
		value = 0x03
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __pulseWR(self, count=1):
		"""Do a negative pulse on the WR pin of the DUT"""
		while count > 0:
			self.__setWR(0)
			self.__setWR(1)
			count -= 1

	def __setBS1(self, high):
		"""Set the BS1 pin of the DUT"""
		value = 0x04
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __setXA0(self, high):
		"""Set the XA0 pin of the DUT"""
		value = 0x05
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __setXA1(self, high):
		"""Set the XA1 pin of the DUT"""
		value = 0x06
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __setXTAL1(self, high):
		"""Set the XTAL1 pin of the DUT"""
		value = 0x07
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __pulseXTAL1(self, count=1):
		"""Do a positive pulse on the XTAL1 pin of the DUT"""
		while count > 0:
			self.__setXTAL1(1)
			self.__setXTAL1(0)
			count -= 1

	def __setPAGEL(self, high):
		"""Set the PAGEL pin of the DUT"""
		value = 0x09
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __pulsePAGEL(self, count=1):
		"""Do a positive pulse on the PAGEL pin of the DUT"""
		while count > 0:
			self.__setPAGEL(1)
			self.__setPAGEL(0)
			count -= 1

	def __setBS2(self, high):
		"""Set the BS2 pin of the DUT"""
		value = 0x0A
		if high:
			value |= 0x80
		self.top.cmdFPGAWrite(0x12, value)

	def __setVoltageControl(self, VPP_en, VPP, VCC_en, VCC):
		value = 0
		if VPP_en:
			value |= 0x01
		if VPP:
			value |= 0x02
		if VCC_en:
			value |= 0x04
		if VCC:
			value |= 0x08
		self.top.cmdFPGAWrite(0x11, value)



"""
#    TOP2049 Open Source programming suite
#
#    Microchip8 - 18f1220 family - 8bit PIC MCU
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chips.microchip8.microchip8_18_common import *

class microchip8_18f1220family(Chip_Microchip8_18_common):
	
	voltageVDD				 = 4.5
	voltageVPP				 = 9
	
	writeBufferSize				 = 8
	eraseBufferSize				 = 64

	def __init__(self,
			chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes
			):
		Chip_Microchip8_18_common.__init__(self,
		chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes)	

	def erase(self):
		self.progressMeterInit("Erasing chip", 0)
		self.enterPM(True)
		self.executeCode(self.getCodeAddrToTBLPTR(0x3C0004))
		self.send4bitWriteInstruction(self.CMD_TW, 0x0080)
		self.executeCode((0x0000,))
		self.sendCommand(1)		
		self.top.cmdDelay(self.delayP11 + self.delayP10)
		for i in range(0,4):
			self.sendCommand(1)
		self.top.flushCommands()
		self.progressMeterFinish()
	
	def setEEPROMAddr(self, addr):
		self.executeCode((0x0E00 | (addr & 0xFF), 0x6EA9))



from pic10f200dip8 import *
from pic10f200sip6 import *
from pic10f202dip8 import *
from pic10f202sip6 import *
from pic10f320dip8 import *
from pic10f320sip6 import *
from pic10f322dip8 import *
from pic10f322sip6 import *
from pic12f1501dip8 import *
from pic12f1501sip6 import *
from pic12f1822dip8 import *
from pic12f1822sip6 import *
from pic12f508dip8 import *
from pic12f508sip6 import *
from pic12f629dip8 import *
from pic12f629sip6 import *
from pic16f1454dip14 import *
from pic16f1454sip6 import *
from pic16f1459dip20 import *
from pic16f1459sip6 import *
from pic16f1503dip14 import *
from pic16f1503sip6 import *
from pic16f1507dip20 import *
from pic16f1507sip6 import *
from pic16f1508dip20 import *
from pic16f1508sip6 import *
from pic16f1509dip20 import *
from pic16f1509sip6 import *
from pic16f1824dip14 import *
from pic16f1824sip6 import *
from pic16f1825dip14 import *
from pic16f1825sip6 import *
from pic16f1826dip18 import *
from pic16f1826sip6 import *
from pic16f1827dip18 import *
from pic16f1827sip6 import *
from pic16f1828dip20 import *
from pic16f1828sip6 import *
from pic16f1829dip20 import *
from pic16f1829sip6 import *
from pic16f1933dip28 import *
from pic16f1933sip6 import *
from pic16f1934dip40 import *
from pic16f1934sip6 import *
from pic16f1936dip28 import *
from pic16f1936sip6 import *
from pic16f1937dip40 import *
from pic16f1937sip6 import *
from pic16f1938dip28 import *
from pic16f1938sip6 import *
from pic16f1939dip40 import *
from pic16f1939sip6 import *
from pic16f59dip40 import *
from pic16f59sip6 import *
from pic16f630dip14 import *
from pic16f630sip6 import *
from pic16f84adip18 import *
from pic16f84asip6 import *
from pic16lf1902dip28 import *
from pic16lf1902sip6 import *
from pic16lf1903dip28 import *
from pic16lf1903sip6 import *
from pic16lf1904dip40 import *
from pic16lf1904sip6 import *
from pic16lf1906dip28 import *
from pic16lf1906sip6 import *
from pic16lf1907dip40 import *
from pic16lf1907sip6 import *
from pic18f1220dip18 import *
from pic18f1220sip6 import *
from pic18f2320dip28 import *
from pic18f2320sip6 import *
from pic18f2321dip28 import *
from pic18f2321sip6 import *
#from pic18f67j60sip6 import *



"""
#    TOP2049 Open Source programming suite
#
#    Atmel Mega8 DIP28 support
#
#    Copyright (c) 2009-2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from atmega_common import *


class Chip_ATMega8DIP28(Chip_ATMega_common):
	def __init__(self):
		Chip_ATMega_common.__init__(self,
			chipPackage = "DIP28",
			chipPinVCC = 7,
			chipPinsVPP = 1,
			chipPinGND = 8,
			signature = "\x1E\x93\x07",
			flashPageSize = 32,
			flashPages = 128,
			eepromPageSize = 4,
			eepromPages = 128,
			fuseBytes = 2)

fuseDesc = (
	BitDescription(0, "CKSEL0"),
	BitDescription(1, "CKSEL1"),
	BitDescription(2, "CKSEL2"),
	BitDescription(3, "CKSEL3"),
	BitDescription(4, "SUT0"),
	BitDescription(5, "SUT1"),
	BitDescription(6, "BODEN"),
	BitDescription(7, "BODLEVEL"),
	BitDescription(8, "BOOTRST"),
	BitDescription(9, "BOOTSZ0"),
	BitDescription(10, "BOOTSZ1"),
	BitDescription(11, "EESAVE"),
	BitDescription(12, "CKOPT"),
	BitDescription(13, "SPIEN"),
	BitDescription(14, "WDTON"),
	BitDescription(15, "RSTDISBL"),
)

lockbitDesc = (
	BitDescription(0, "LB1"),
	BitDescription(1, "LB2"),
	BitDescription(2, "BLB01"),
	BitDescription(3, "BLB02"),
	BitDescription(4, "BLB11"),
	BitDescription(5, "BLB12"),
	BitDescription(6, "Unused"),
	BitDescription(7, "Unused"),
	BitDescription(8, "Unused"),
)

ChipDescription(
	Chip_ATMega8DIP28,
	bitfile = "atmega8dip28",
	runtimeID = (0x0003, 0x01),
	chipVendors = "Atmel",
	description = "AtMega8",
	fuseDesc = fuseDesc,
	lockbitDesc = lockbitDesc,
	packages = ( ("DIP28", ""), )
)



"""
#    TOP2049 Open Source programming suite
#
#    TOP2049 Lowlevel hardware access
#
#    Copyright (c) 2012 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.util import *
from libtoprammer.hardware_access_usb import *


class HardwareAccess(HardwareAccessUSB):
	"TOP2049 hardware access"

	ADDR_OK_BIT	= 4

	def __init__(self, foundUSBDev,
		     noQueue=False, doRawDump=False):
		HardwareAccessUSB.__init__(self,
			usbdev = foundUSBDev.usbdev,
			maxPacketBytes = 64,
			noQueue = noQueue,
			doRawDump = doRawDump)

	def getOscillatorHz(self):
		"Get the 'OSC' frequency."
		return 24000000

	def getBufferRegSize(self):
		"Get the buffer register size, in bytes."
		return 64

	def readBufferReg(self, nrBytes):
		"Reads and returns the buffer register from hardware."
		self.queueCommand(int2byte(0x07))
		return self.receive(self.getBufferRegSize())[:nrBytes]

	def hardwareInit(self):
		"Initialize the hardware."
		self.queueCommand(b"\x0D")
		if self.readBufferReg(4) != b"\x69\x0C\x02\x00":
			print("Init: Unexpected status (a)")

		self.setVPPVoltage(0)
		self.setVPPVoltage(0)
		self.queueCommand(b"\x0E\x20\x00\x00")
		self.delay(0.01)
		self.setVCCVoltage(0)

		self.loadGNDLayout(0)
		self.loadVPPLayout(0)
		self.loadVCCLayout(0)

		self.queueCommand(b"\x0E\x20\x00\x00")
		self.delay(0.01)
		self.queueCommand(b"\x0E\x25\x00\x00")
		if self.readBufferReg(4) != b"\x6C\x68\x00\x00":
			print("Init: Unexpected status (b)")
		self.enableZifPullups(False)
		self.flushCommands()

	def readVersionString(self):
		"Returns the device ID and versioning string."
		self.queueCommand(b"\x0E\x11\x00\x00")
		data = self.readBufferReg(16)
		return data.strip()

	def getFPGAType(self):
		"Get the FPGA architecture."
		return "xc2s15"

	def getFPGAMaxConfigChunkSize(self):
		"Maximum config chunk size."
		return 60

	def FPGAInitiateConfig(self):
		"Initiate a configuration sequence on the FPGA."
		self.queueCommand(b"\x0E\x21\x00\x00")
		stat = byte2int(self.readBufferReg(1))
		expected = 0x01
		if stat != expected:
			raise TOPException("bit-upload: Failed to initiate " +\
				"config sequence (got 0x%02X, expected 0x%02X)" %\
				(stat, expected))

	def FPGAUploadConfig(self, offset, data):
		"Upload configuration data into the FPGA."
		assert(len(data) <= self.getFPGAMaxConfigChunkSize())
		cmd = b"\x0E\x22\x00\x00" + data
		cmd += b"\x00" * (64 - len(cmd)) # padding
		self.queueCommand(cmd)

	def makeFPGAAddr(self, address):
		# Set the "address OK" bit
		return address | (1 << self.ADDR_OK_BIT)

	def FPGARead(self, address):
		"Do an FPGA read at 'address'. Data is put into buffer reg."
		address = self.makeFPGAAddr(address)
		if address == self.makeFPGAAddr(0): # Fast tracked
			self.queueCommand(int2byte(0x01))
			return
		self.queueCommand(int2byte(0x0B) + int2byte(address))

	def FPGAWrite(self, address, byte):
		"Write 'byte' to FPGA at 'address'."
		address = self.makeFPGAAddr(address)
		if address == self.makeFPGAAddr(0): # Fast tracked
			self.queueCommand(int2byte(0x10) + int2byte(byte))
			return
		self.queueCommand(int2byte(0x0A) + int2byte(address) +\
				  int2byte(byte))

	def loadGNDLayout(self, layout):
		"Load the GND configuration into the H/L shiftregisters."
		cmd = int2byte(0x0E) + int2byte(0x16) +\
		      int2byte(layout) + int2byte(0)
		self.queueCommand(cmd)
		self.delay(0.01)
		self.flushCommands(0.15)

	def setVPPVoltage(self, voltage):
		"Set the VPP voltage. voltage is a floating point voltage number."
		centivolt = int(voltage * 10)
		cmd = int2byte(0x0E) + int2byte(0x12) +\
		      int2byte(centivolt) + int2byte(0)
		self.queueCommand(cmd)
		self.delay(0.01)

	def loadVPPLayout(self, layout):
		"Load the VPP configuration into the shift registers."
		cmd = int2byte(0x0E) + int2byte(0x14) +\
		      int2byte(layout) + int2byte(0)
		self.queueCommand(cmd)
		self.delay(0.01)
		self.flushCommands(0.15)

	def setVCCVoltage(self, voltage):
		"Set the VCC voltage."
		centivolt = int(voltage * 10)
		cmd = int2byte(0x0E) + int2byte(0x13) +\
		      int2byte(centivolt) + int2byte(0)
		self.queueCommand(cmd)
		self.delay(0.01)

	def loadVCCLayout(self, layout):
		"Load the VCC configuration into the shift registers."
		cmd = int2byte(0x0E) + int2byte(0x15) +\
		      int2byte(layout) + int2byte(0)
		self.queueCommand(cmd)
		self.delay(0.01)
		self.flushCommands(0.15)

	def enableZifPullups(self, enable):
		"Enable the ZIF socket signal pullups."
		param = 1 if enable else 0
		cmd = int2byte(0x0E) + int2byte(0x28) +\
		      int2byte(param) + int2byte(0)
		self.queueCommand(cmd)

	def __delay_4usec(self):
		self.queueCommand(int2byte(0x00))

	def __delay_10msec(self):
		self.queueCommand(int2byte(0x1B))

	def delay(self, seconds):
		"Perform an on-device or host delay."
		if seconds >= 0.5:
			# Perform long delays on the host
			self.flushCommands(seconds)
			return
		if seconds > 0.000255:
			# Need to round up to ten milliseconds
			millisecs = int(math.ceil(seconds * 1000))
			millisecs = roundup(millisecs, 10)
			for i in range(0, millisecs // 10):
				self.__delay_10msec()
		else:
			# Round up to 4 usec boundary
			microsecs = int(math.ceil(seconds * 1000000))
			microsecs = roundup(microsecs, 4)
			for i in range(0, microsecs // 4):
				self.__delay_4usec()



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC10F322 DIP8
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_splittedPMarea_hasResetPC import *

class Chip_Pic10F322dip8(microchip8_splittedPMarea_hasResetPC):
	
	rowSize = 16
	nLatches = 16
	
	logicalFlashProgramMemorySize = 0x2000
	logicalFlashConfigurationMemorySize = 0x2000

    	def __init__(self):
	    	microchip8_splittedPMarea_hasResetPC.__init__(self,
			chipPackage="DIP8",
			chipPinVCC=2,
			chipPinsVPP=8,
			chipPinGND=7,
			signature="\x82\x29",
			flashPageSize=0x200,
			flashPages=1,
			eepromPageSize=0,
			eepromPages=0,
			fuseBytes=2
			)

fuseDesc = (
	BitDescription(0, "FOSC, 1=CLKIN, 0=internal"),
	BitDescription(1, "BOREN[0]"),
	BitDescription(2, "BOREN[1]"),
	BitDescription(3, "WDTE[0]"),
	BitDescription(4, "WDTE[1]"),
	BitDescription(5, "nPWRTE"),
	BitDescription(6, "MCLRE, 1=RA3 is nMCLR, weak pull-up enabled"),
	BitDescription(7, "nCP"),
	BitDescription(8, "LVP"),
	BitDescription(9, "LPBOREN"),
	BitDescription(10, "BORV"),
	BitDescription(11, "WRT[0]"),
	BitDescription(12, "WRT[1] 11=write protection off"),
	BitDescription(13, "Unused"),
)

ChipDescription(
	Chip_Pic10F322dip8,
	bitfile="pic10fxxxdip8",
	chipID="pic10f322dip8",
	runtimeID=(0xDE01, 0x01),
	chipVendors="Microchip",
	description="PIC10F322, PIC10LF322",
	packages=(("DIP8", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



#!/usr/bin/env python
"""
#    TOP2049 Open Source programming suite
#
#    USB dump parser
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import re

packethdr_re = re.compile(r"Dev\s+([0-9a-fA-F]{4,4}):([0-9a-fA-F]{4,4})\s+EP([0-9a-fA-F]+)\s+(\w+)\s(\w+)\s+len=(\d+)")
payload_re = re.compile(r"\[([0-9a-fA-F]+)\]:\s+([0-9a-fA-F\s]+)\s+.+")


def generateHexdump(mem):
	def toAscii(char):
		if char >= 32 and char <= 126:
			return chr(char)
		return "."

	ret = ""
	ascii = ""
	for i in range(0, len(mem)):
		if i % 16 == 0 and i != 0:
			ret += "  " + ascii + "\n"
			ascii = ""
		if i % 16 == 0:
			ret += "0x%04X:  " % i
		c = mem[i]
		ret += "%02X" % c
		if (i % 2 != 0):
			ret += " "
		ascii += toAscii(c)
	ret += "  " + ascii + "\n\n"
	return ret

def dumpMem(mem):
	sys.stdout.write(generateHexdump(mem))

def dumpInstr(instr, description):
	for i in instr:
		sys.stdout.write("%02X" % i)
	sys.stdout.write("  " * (10 - len(instr)))
	sys.stdout.write(": " + description)
	sys.stdout.write("\n")

def parseBulkIn(data):
	if len(data) == 64:
		print "Read buffer register"
		dumpMem(data)

def parseBulkOut(data):
	i = 0
	while i < len(data):
		if data[i] == 0x00:
			dumpInstr(data[i:i+1], "Delay 4 usec")
		elif data[i] == 0x01:
			dumpInstr(data[i:i+1], "Read byte from FPGA")
		elif data[i] == 0x07:
			dumpInstr(data[i:i+1], "Read buffer register request")
		elif data[i] == 0x0A:
			dumpInstr(data[i:i+3], "Write 0x%02X to the FPGA at address 0x%02X" % (data[i+2], data[i+1]))
			i += 2
		elif data[i] == 0x0B:
			dumpInstr(data[i:i+2], "Read data from FPGA at address 0x%02X" % (data[i+1]))
			i += 1
		elif data[i] == 0x0E and data[i+1] == 0x11:
			dumpInstr(data[i:i+4], "Read device ID request")
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x12:
			centivolts = data[i+2]
			dumpInstr(data[i:i+4], "Set VPP to %.2f Volts" % (float(centivolts) / 10))
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x13:
			centivolts = data[i+2]
			dumpInstr(data[i:i+4], "Set VCC to %.2f Volts" % (float(centivolts) / 10))
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x14:
			dumpInstr(data[i:i+4], "Loading VPP layout %d" % data[i+1])
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x15:
			dumpInstr(data[i:i+4], "Loading VCC layout %d" % data[i+1])
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x16:
			dumpInstr(data[i:i+4], "Loading GND layout %d" % data[i+1])
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x20:
			dumpInstr(data[i:i+4], "Unknown 0x0E20%02X%02X" % (data[i+2], data[i+3]))
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x21:
			dumpInstr(data[i:i+4], "Initiate FPGA programming")
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x22:
			dumpInstr(data[i:i+4], "Upload FPGA configuration data")
			i += 63
		elif data[i] == 0x0E and data[i+1] == 0x25:
			dumpInstr(data[i:i+4], "Unknown 0x0E25")
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x28:
			op = "Disable"
			if data[i+2] == chr(0):
				op = "Enable"
			dumpInstr(data[i:i+4], "%s the ZIF socket" % op)
			i += 3
		elif data[i] == 0x0E and data[i+1] == 0x1F:
			dumpInstr(data[i:i+4], "Unknown 0x0E1F")
			i += 3
		elif data[i] == 0x0D:
			dumpInstr(data[i:i+1], "Unknown 0x0D")
		elif data[i] == 0x10:
			dumpInstr(data[i:i+2], "Write 0x%02X to the FPGA at address 0x10" % data[i+1])
			i += 1
		elif data[i] == 0x19:
			dumpInstr(data[i:i+1], "Unknown 0x19")
		elif data[i] == 0x1B:
			dumpInstr(data[i:i+1], "Delay 10 msec")
		elif data[i] == 0x34:
			dumpInstr(data[i:i+1], "Unknown 0x34")
		elif data[i] == 0x38:
			dumpInstr(data[i:i+2], "Unknown 0x38")
			i += 1
		elif data[i] == 0x39:
			dumpInstr(data[i:i+1], "Unknown 0x39")
		elif data[i] == 0x4A:
			dumpInstr(data[i:i+2], "Unknown 0x4A")
			i += 1
		elif data[i] == 0x4B:
			dumpInstr(data[i:i+2], "Unknown 0x4B")
			i += 1
		else:
			print "UNKNOWN INSTRUCTION 0x%02X. Aborting..." % data[i]
			for j in range(i, len(data)):
				sys.stdout.write("%02X " % data[j])
			print ""
			sys.exit(1)
		i += 1

def parseDumpFile(fd):
	transtype = None
	bulkData = []
	for line in fd.readlines():
		if transtype == "BULK":
			# Bulk IN or OUT transfer
			m = payload_re.match(line)
			if m:
				offset = int(m.group(1), 16)
				dataString = m.group(2)

				dataString = dataString.replace(" ", "").strip()
				assert(len(dataString) % 2 == 0)
				for i in range(0, len(dataString), 2):
					byteStr = dataString[i:i+2]
					bulkData.append(int(byteStr, 16))
			else:
				# Transfer done
				assert(len(bulkData) == length)
				if direction == "IN":
					parseBulkIn(bulkData)
				elif direction == "OUT":
					parseBulkOut(bulkData)
				else:
					assert(0)
				transtype = None
				bulkData = []

		m = packethdr_re.match(line)
		if m:
			vendor = int(m.group(1), 16)
			device = int(m.group(2), 16)
			ep = int(m.group(3), 16)
			transtype = m.group(4).upper()
			direction = m.group(5).upper()
			length = int(m.group(6))

def usage():
	print "dump-parser.py file.dump"

def main(argv):
	if len(argv) != 2:
		usage()
		return 1

	fd = open(argv[1])
	parseDumpFile(fd)

	return 0

if __name__ == "__main__":
	sys.exit(main(sys.argv))



"""
#    TOP2049 Open Source programming suite
#
#    configWordsSet - configWord specification for 16bit PIC MCUs
#
#    Copyright (c) 2014 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from  libtoprammer.chip import BitDescription

ka200_fuseDesc = (
	BitDescription(0o00, "NA"),
	BitDescription(0o01, "NA"),
	BitDescription(0o02, "NA"),
	BitDescription(0o03, "NA"),
	BitDescription(0o04, "NA"),
	BitDescription(0o05, "NA"),
	BitDescription(0o06, "NA"),
	BitDescription(0o07, "NA"),
	BitDescription(0o10, "NA"),
	BitDescription(0o11, "NA"),
	BitDescription(0o12, "NA"),
	BitDescription(0o13, "NA"),
	BitDescription(0o14, "NA"),
	BitDescription(0o15, "NA"),
	BitDescription(0o16, "NA"),
	BitDescription(0o17, "NA"),
	
	BitDescription(0o20, "NA"),
	BitDescription(0o21, "NA"),
	BitDescription(0o22, "NA"),
	BitDescription(0o23, "NA"),
	BitDescription(0o24, "NA"),
	BitDescription(0o25, "NA"),
	BitDescription(0o26, "NA"),
	BitDescription(0o27, "NA"),
	BitDescription(0o30, "NA"),
	BitDescription(0o31, "NA"),
	BitDescription(0o32, "NA"),
	BitDescription(0o33, "NA"),
	BitDescription(0o34, "NA"),
	BitDescription(0o35, "NA"),
	BitDescription(0o36, "NA"),
	BitDescription(0o37, "NA"),
	
	BitDescription(0o40, "FGS:GWRP: General Segment Code Flash Write Protection bit, 1 = General segment may be written"),
	BitDescription(0o41, "FGS:GSS0 General Segment Code Flash Code Protection bit, 1 = no protection"),
	BitDescription(0o42, "NA"),
	BitDescription(0o43, "NA"),
	BitDescription(0o44, "NA"),
	BitDescription(0o45, "NA"),
	BitDescription(0o46, "NA"),
	BitDescription(0o47, "NA"),
	BitDescription(0o50, "NA"),
	BitDescription(0o51, "NA"),
	BitDescription(0o52, "NA"),
	BitDescription(0o53, "NA"),
	BitDescription(0o54, "NA"),
	BitDescription(0o55, "NA"),
	BitDescription(0o56, "NA"),
	BitDescription(0o57, "NA"),
	
	BitDescription(0o60, "FOSCSEL: FNOSC[0] 101 = 500 kHz Low-Power FRC oscillato"),
	BitDescription(0o61, "FOSCSEL: FNOSC[1] 110 = 500 kHz Low-Power FRC oscillator with divide-by-N (LPFRCDIV)"),
	BitDescription(0o62, "FOSCSEL: FNOSC[2] 111 = 8 MHz FRC oscillator with divide-by-N (FRCDIV)"),
	BitDescription(0o63, "NA"),
	BitDescription(0o64, "NA"),
	BitDescription(0o65, "NA"),
	BitDescription(0o66, "NA"),
	BitDescription(0o67, "FOSCSEL: IESO: Internal External Switchover bit, 1 = enabled"),
	BitDescription(0o70, "NA"),
	BitDescription(0o71, "NA"),
	BitDescription(0o72, "NA"),
	BitDescription(0o73, "NA"),
	BitDescription(0o74, "NA"),
	BitDescription(0o75, "NA"),
	BitDescription(0o76, "NA"),
	BitDescription(0o77, "NA"),
	
	BitDescription(0o100, "FOSC: POSCMD[0] 11 = Primary oscillator disabled = HS Oscillator mode selected"),
	BitDescription(0o101, "FOSC: POSCMD[1]"),
	BitDescription(0o102, "FOSC: OSCIOFNC 1 = CLKO output signal active on the OSCO pin"),
	BitDescription(0o103, "FOSC: POSCFREQ[0] 00 = Reserved; do not use"),
	BitDescription(0o104, "FOSC: POSCFREQ[1] 11 = Primary oscillator/external clock input frequency greater than 8 MHz"),
	BitDescription(0o105, "FOSC: SOSCSEL 1 = Secondary oscillator configured for high-power operation"),
	BitDescription(0o106, "FOSC: FCKSM[0] 00 = Clock switching is enabled, Fail-Safe Clock Monitor is enabled"),
	BitDescription(0o107, "FOSC: FCKSM[1] 1x = Clock switching is disabled, Fail-Safe Clock Monitor is disabled"),
	BitDescription(0o110, "NA"),
	BitDescription(0o111, "NA"),
	BitDescription(0o112, "NA"),
	BitDescription(0o113, "NA"),
	BitDescription(0o114, "NA"),
	BitDescription(0o115, "NA"),
	BitDescription(0o116, "NA"),
	BitDescription(0o117, "NA"), 	
    
	BitDescription(0o120, "FWDT: WDTPS[0] 0000 = 1:1"),
	BitDescription(0o121, "FWDT: WDTPS[1] 0001 = 1:2"),
	BitDescription(0o122, "FWDT: WDTPS[2] 0010 = 1:4"),
	BitDescription(0o123, "FWDT: WDTPS[3] 1111 = 1:32768"),
	BitDescription(0o124, "FWDT: FWPSA 1 = WDT prescaler ratio of 1:128"),
	BitDescription(0o125, "NA"),
	BitDescription(0o126, "FWDT: WINDIS: 1 = Standard WDT selected; windowed WDT disabled"),
	BitDescription(0o127, "FWDT: FWDTEN: 1 = WDT enabled"),
	BitDescription(0o130, "NA"),
	BitDescription(0o131, "NA"),
	BitDescription(0o132, "NA"),
	BitDescription(0o133, "NA"),
	BitDescription(0o134, "NA"),
	BitDescription(0o135, "NA"),
	BitDescription(0o136, "NA"),
	BitDescription(0o137, "NA"),
	
	BitDescription(0o140, "FPOR: BOREN[0] 00 = Brown-out Reset disabled in hardware; SBOREN bit disabled"),
	BitDescription(0o141, "FPOR: BOREN[1] 11 = Brown-out Reset enabled in hardware; SBOREN bit disabled"),
	BitDescription(0o142, "NA"),
	BitDescription(0o143, "FPOR: PWRTEN: 1 = Power-up Timer Enable bit enabled"),
	BitDescription(0o144, "NA"),
	BitDescription(0o145, "FPOR: BORV[0] 00 = Low-power Brown-out Reset occurs around 2.0V"),
	BitDescription(0o146, "FPOR: BORV[1] 11 = Brown-out Reset set to lowest voltage"),
	BitDescription(0o147, "FPOR: MCLRE: 1 = MCLR pin enabled; RA5 input pin disabled"),
	BitDescription(0o150, "NA"),
	BitDescription(0o151, "NA"),
	BitDescription(0o152, "NA"),
	BitDescription(0o153, "NA"),
	BitDescription(0o154, "NA"),
	BitDescription(0o155, "NA"),
	BitDescription(0o156, "NA"),
	BitDescription(0o157, "NA"), 	
	
	BitDescription(0o160, "FICD[0] 10 = PGC2/PGD2 are used for programming the device"),
	BitDescription(0o161, "FICD[1] 00, 11 = Reserved; do not use"),
	BitDescription(0o162, "NA"),
	BitDescription(0o163, "NA"),
	BitDescription(0o164, "NA"),
	BitDescription(0o165, "NA"),
	BitDescription(0o166, "NA"),
	BitDescription(0o167, "NA"),
	BitDescription(0o170, "NA"),
	BitDescription(0o171, "NA"),
	BitDescription(0o172, "NA"),
	BitDescription(0o173, "NA"),
	BitDescription(0o174, "NA"),
	BitDescription(0o175, "NA"),
	BitDescription(0o176, "NA"),
	BitDescription(0o137, "NA"),
)
klx0x_fuseDesc = (
	BitDescription(0o00, "FBS:BWRP 1 = Boot Segment may be written"),
	BitDescription(0o01, "FBS:BSS[0] 001 = High-security Boot Segment starts at 0200h, ends at 15FEh"),
	BitDescription(0o02, "FBS:BSS[1] 101 = Standard security Boot Segment starts at 0200h, ends at 15FEh"),
	BitDescription(0o03, "FBS:BSS[2] 111 = No Boot Segment; all program memory space is General Segment"),
	BitDescription(0o04, "NA"),
	BitDescription(0o05, "NA"),
	BitDescription(0o06, "NA"),
	BitDescription(0o07, "NA"),
	BitDescription(0o10, "NA"),
	BitDescription(0o11, "NA"),
	BitDescription(0o12, "NA"),
	BitDescription(0o13, "NA"),
	BitDescription(0o14, "NA"),
	BitDescription(0o15, "NA"),
	BitDescription(0o16, "NA"),
	BitDescription(0o17, "NA"),
	
	BitDescription(0o20, "NA"),
	BitDescription(0o21, "NA"),
	BitDescription(0o22, "NA"),
	BitDescription(0o23, "NA"),
	BitDescription(0o24, "NA"),
	BitDescription(0o25, "NA"),
	BitDescription(0o26, "NA"),
	BitDescription(0o27, "NA"),
	BitDescription(0o30, "NA"),
	BitDescription(0o31, "NA"),
	BitDescription(0o32, "NA"),
	BitDescription(0o33, "NA"),
	BitDescription(0o34, "NA"),
	BitDescription(0o35, "NA"),
	BitDescription(0o36, "NA"),
	BitDescription(0o37, "NA"),
	
	BitDescription(0o40, "FGS:GWRP: General Segment Code Flash Write Protection bit, 1 = General segment may be written"),
	BitDescription(0o41, "FGS:GSS0 General Segment Code Flash Code Protection bit, 1 = no protection"),
	BitDescription(0o42, "NA"),
	BitDescription(0o43, "NA"),
	BitDescription(0o44, "NA"),
	BitDescription(0o45, "NA"),
	BitDescription(0o46, "NA"),
	BitDescription(0o47, "NA"),
	BitDescription(0o50, "NA"),
	BitDescription(0o51, "NA"),
	BitDescription(0o52, "NA"),
	BitDescription(0o53, "NA"),
	BitDescription(0o54, "NA"),
	BitDescription(0o55, "NA"),
	BitDescription(0o56, "NA"),
	BitDescription(0o57, "NA"),
	
	BitDescription(0o60, "FOSCSEL: FNOSC[0] 101 = 000 = 8 MHz FRC Oscillator (FRC)"),
	BitDescription(0o61, "FOSCSEL: FNOSC[1] 110 = 500 kHz Low-Power FRC oscillator with divide-by-N (LPFRCDIV)"),
	BitDescription(0o62, "FOSCSEL: FNOSC[2] 111 = 8 MHz FRC oscillator with divide-by-N (FRCDIV)"),
	BitDescription(0o63, "NA"),
	BitDescription(0o64, "NA"),
	BitDescription(0o65, "FOSCSEL: SOSCSRC 1 = SOSC analog crystal function is available on the SOSCI/SOSCO pins"),
	BitDescription(0o66, "FOSCSEL: LPRCSEL 1 = High-Power/High-Accuracy mode"),
	BitDescription(0o67, "FOSCSEL: IESO: Internal External Switchover bit, 1 = enabled"),
	BitDescription(0o70, "NA"),
	BitDescription(0o71, "NA"),
	BitDescription(0o72, "NA"),
	BitDescription(0o73, "NA"),
	BitDescription(0o74, "NA"),
	BitDescription(0o75, "NA"),
	BitDescription(0o76, "NA"),
	BitDescription(0o77, "NA"),
	
	BitDescription(0o100, "FOSC: POSCMD[0] 11 = Primary oscillator disabled = HS Oscillator mode selected"),
	BitDescription(0o101, "FOSC: POSCMD[1]"),
	BitDescription(0o102, "FOSC: OSCIOFNC 1 = CLKO output signal active on the OSCO pin"),
	BitDescription(0o103, "FOSC: POSCFREQ[0] 00 = Reserved; do not use"),
	BitDescription(0o104, "FOSC: POSCFREQ[1] 11 = Primary oscillator/external clock input frequency greater than 8 MHz"),
	BitDescription(0o105, "FOSC: SOSCSEL 1 = Secondary oscillator configured for high-power operation"),
	BitDescription(0o106, "FOSC: FCKSM[0] 00 = Clock switching is enabled, Fail-Safe Clock Monitor is enabled"),
	BitDescription(0o107, "FOSC: FCKSM[1] 1x = Clock switching is disabled, Fail-Safe Clock Monitor is disabled"),
	BitDescription(0o110, "NA"),
	BitDescription(0o111, "NA"),
	BitDescription(0o112, "NA"),
	BitDescription(0o113, "NA"),
	BitDescription(0o114, "NA"),
	BitDescription(0o115, "NA"),
	BitDescription(0o116, "NA"),
	BitDescription(0o117, "NA"), 	
    
	BitDescription(0o120, "FWDT: WDTPS[0] 0000 = 1:1"),
	BitDescription(0o121, "FWDT: WDTPS[1] 0001 = 1:2"),
	BitDescription(0o122, "FWDT: WDTPS[2] 0010 = 1:4"),
	BitDescription(0o123, "FWDT: WDTPS[3] 1111 = 1:32768"),
	BitDescription(0o124, "FWDT: FWPSA 1 = WDT prescaler ratio of 1:128"),
	BitDescription(0o125, "FWDT: FWDTEN[0]: 00 = WDT is disabled in hardware; SWDTEN bit is disabled"),
	BitDescription(0o126, "FWDT: WINDIS: 1 = Standard WDT selected; windowed WDT disabled"),
	BitDescription(0o127, "FWDT: FWDTEN[1]: 11 = WDT is enabled in hardware"),
	BitDescription(0o130, "NA"),
	BitDescription(0o131, "NA"),
	BitDescription(0o132, "NA"),
	BitDescription(0o133, "NA"),
	BitDescription(0o134, "NA"),
	BitDescription(0o135, "NA"),
	BitDescription(0o136, "NA"),
	BitDescription(0o137, "NA"),
	
	BitDescription(0o140, "FPOR: BOREN[0] 00 = Brown-out Reset disabled in hardware; SBOREN bit disabled"),
	BitDescription(0o141, "FPOR: BOREN[1] 11 = Brown-out Reset enabled in hardware; SBOREN bit disabled"),
	BitDescription(0o142, "NA"),
	BitDescription(0o143, "FPOR: PWRTEN: 1 = Power-up Timer Enable bit enabled"),
	BitDescription(0o144, "FPOR: I2C1SEL: Alternate MSSP1 I2CTM Pin Mapping bit 1 = default loc."),
	BitDescription(0o145, "FPOR: BORV[0] 00 = Low-power Brown-out Reset occurs around 2.0V"),
	BitDescription(0o146, "FPOR: BORV[1] 11 = Brown-out Reset set to lowest voltage"),
	BitDescription(0o147, "FPOR: MCLRE: 1 = MCLR pin enabled; RA5 input pin disabled"),
	BitDescription(0o150, "NA"),
	BitDescription(0o151, "NA"),
	BitDescription(0o152, "NA"),
	BitDescription(0o153, "NA"),
	BitDescription(0o154, "NA"),
	BitDescription(0o155, "NA"),
	BitDescription(0o156, "NA"),
	BitDescription(0o157, "NA"), 	
	
	BitDescription(0o160, "FICD: ICS[0] 11 = PGEC1/PGED1 are used for programming and debugging the device"),
	BitDescription(0o161, "FICD: ICS[1] 00, 11 = Reserved; do not use"),
	BitDescription(0o162, "NA"),
	BitDescription(0o163, "NA"),
	BitDescription(0o164, "NA"),
	BitDescription(0o165, "NA"),
	BitDescription(0o166, "NA"),
	BitDescription(0o167, "FICD: DEBUG 1 = Background debugger is disabled"),
	BitDescription(0o170, "NA"),
	BitDescription(0o171, "NA"),
	BitDescription(0o172, "NA"),
	BitDescription(0o173, "NA"),
	BitDescription(0o174, "NA"),
	BitDescription(0o175, "NA"),
	BitDescription(0o176, "NA"),
	BitDescription(0o137, "NA"),
)



"""
#    TOP2049 Open Source programming suite
#
#    Atmel Tiny13 DIP8
#
#    Copyright (c) 2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_AtTiny13dip8(Chip):
	PROGCMD_SENDINSTR	= 1 # Send an instruction to the chip

	STAT_BUSY		= 0x01 # Programmer is running a command
	STAT_SDO		= 0x02 # Raw SDO pin state

	def __init__(self):
		Chip.__init__(self,
			      chipPackage = "DIP8",
			      chipPinVCC = 8,
			      chipPinsVPP = 1,
			      chipPinGND = 4)
		self.signature = "\x1E\x90\x07"
		self.flashPageSize = 16
		self.flashPages = 32
		self.eepromPageSize = 4
		self.eepromPages = 16

	def readSignature(self):
		self.__enterPM()
		self.progressMeterInit("Reading signature", 0)
		signature = self.__readSignature()
		self.progressMeterFinish()
		return signature

	def erase(self):
		self.__enterPM()
		self.progressMeterInit("Erasing chip", 0)
		self.__sendInstr(SDI=0x80, SII=0x4C)
		self.__sendInstr(SDI=0x00, SII=0x64)
		self.__sendInstr(SDI=0x00, SII=0x6C)
		self.__waitHighSDO()
		self.__sendNOP()
		self.progressMeterFinish()

	def readProgmem(self):
		nrWords = self.flashPages * self.flashPageSize
		image = ""
		self.__enterPM()
		self.progressMeterInit("Reading flash", nrWords)
		self.__sendReadFlashInstr()
		currentHigh = -1
		bufferedBytes = 0
		for word in range(0, nrWords):
			self.progressMeter(word)
			low = word & 0xFF
			high = (word >> 8) & 0xFF
			self.__sendInstr(SDI=low, SII=0x0C)
			if high != currentHigh:
				self.__sendInstr(SDI=high, SII=0x1C)
				currentHigh = high
			self.__sendInstr(SDI=0x00, SII=0x68)
			self.__sendInstr(SDI=0x00, SII=0x6C)
			self.__readSDOBufferHigh()
			bufferedBytes += 1
			self.__sendInstr(SDI=0x00, SII=0x78)
			self.__sendInstr(SDI=0x00, SII=0x7C)
			self.__readSDOBufferHigh()
			bufferedBytes += 1
			if bufferedBytes == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(bufferedBytes)
				bufferedBytes = 0
		image += self.top.cmdReadBufferReg(bufferedBytes)
		self.progressMeterFinish()
		return image

	def writeProgmem(self, image):
		nrWords = self.flashPages * self.flashPageSize
		if len(image) > nrWords * 2 or len(image) % 2 != 0:
			self.throwError("Invalid flash image size %d (expected <=%d and word aligned)" %\
				(len(image), nrWords * 2))
		self.__enterPM()
		self.progressMeterInit("Writing flash", len(image) // 2)
		self.__sendWriteFlashInstr()
		currentHigh = -1
		for word in range(0, len(image) // 2):
			self.progressMeter(word)
			low = word & 0xFF
			high = (word >> 8) & 0xFF
			self.__sendInstr(SDI=low, SII=0x0C)
			self.__sendInstr(SDI=byte2int(image[word * 2 + 0]), SII=0x2C)
			self.__sendInstr(SDI=byte2int(image[word * 2 + 1]), SII=0x3C)
			self.__sendInstr(SDI=0x00, SII=0x7D)
			self.__sendInstr(SDI=0x00, SII=0x7C)
			if ((word + 1) % self.flashPageSize == 0) or word == len(image) // 2 - 1:
				if currentHigh != high:
					self.__sendInstr(SDI=high, SII=0x1C)
					currentHigh = high
				self.__sendInstr(SDI=0x00, SII=0x64)
				self.__sendInstr(SDI=0x00, SII=0x6C)
				self.__waitHighSDO()
		self.__sendNOP()
		self.progressMeterFinish()

	def readEEPROM(self):
		nrBytes = self.eepromPages * self.eepromPageSize
		image = ""
		self.__enterPM()
		self.progressMeterInit("Reading EEPROM", nrBytes)
		self.__sendReadEEPROMInstr()
		currentPage = -1
		bufferedBytes = 0
		for i in range(0, nrBytes):
			self.progressMeter(i)
			low = i & 0xFF
			high = (i >> 8) & 0xFF
			self.__sendInstr(SDI=low, SII=0x0C)
			if currentPage != high:
				self.__sendInstr(SDI=high, SII=0x1C)
				currentPage = high
			self.__sendInstr(SDI=0x00, SII=0x68)
			self.__sendInstr(SDI=0x00, SII=0x6C)
			self.__readSDOBufferHigh()
			bufferedBytes += 1
			if bufferedBytes == self.top.getBufferRegSize():
				image += self.top.cmdReadBufferReg(bufferedBytes)
				bufferedBytes = 0
		image += self.top.cmdReadBufferReg(bufferedBytes)
		self.progressMeterFinish()
		return image

	def writeEEPROM(self, image):
		nrBytes = self.eepromPages * self.eepromPageSize
		if len(image) > nrBytes:
			self.throwError("Invalid EEPROM image size %d (expected <=%d)" %\
				(len(image), nrBytes))
		self.__enterPM()
		self.progressMeterInit("Writing EEPROM", len(image))
		self.__sendWriteEEPROMInstr()
		for i in range(0, len(image)):
			self.progressMeter(i)
			self.__sendInstr(SDI=i, SII=0x0C)
			self.__sendInstr(SDI=byte2int(image[i]), SII=0x2C)
			self.__sendInstr(SDI=0x00, SII=0x6D)
			self.__sendInstr(SDI=0x00, SII=0x6C)
			if ((i + 1) % self.eepromPageSize == 0) or i == len(image) - 1:
				self.__sendInstr(SDI=0x00, SII=0x64)
				self.__sendInstr(SDI=0x00, SII=0x6C)
				self.__waitHighSDO()
		self.__sendNOP()
		self.progressMeterFinish()

	def readFuse(self):
		fuses = []
		self.__enterPM()
		self.progressMeterInit("Reading fuses", 0)
		self.__sendInstr(SDI=0x04, SII=0x4C)
		self.__sendInstr(SDI=0x00, SII=0x68)
		self.__sendInstr(SDI=0x00, SII=0x6C)
		self.__readSDOBufferHigh()
		fuses.append(self.top.cmdReadBufferReg(1))
		self.__sendInstr(SDI=0x04, SII=0x4C)
		self.__sendInstr(SDI=0x00, SII=0x7A)
		self.__sendInstr(SDI=0x00, SII=0x7E)
		self.__readSDOBufferHigh()
		fuses.append(int2byte(self.top.cmdReadBufferReg8() | 0xE0))
		self.progressMeterFinish()
		return b"".join(fuses)

	def writeFuse(self, image):
		if len(image) != 2:
			self.throwError("Invalid Fuses image size %d (expected %d)" %\
				(len(image), 2))
		self.__enterPM()
		self.progressMeterInit("Writing fuses", 0)
		self.__sendInstr(SDI=0x40, SII=0x4C)
		self.__sendInstr(SDI=byte2int(image[0]), SII=0x2C)
		self.__sendInstr(SDI=0x00, SII=0x64)
		self.__sendInstr(SDI=0x00, SII=0x6C)
		self.__waitHighSDO()
		self.__sendInstr(SDI=0x40, SII=0x4C)
		self.__sendInstr(SDI=(byte2int(image[1]) & 0x1F), SII=0x2C)
		self.__sendInstr(SDI=0x00, SII=0x74)
		self.__sendInstr(SDI=0x00, SII=0x7C)
		self.__waitHighSDO()
		self.progressMeterFinish()

	def readLockbits(self):
		self.__enterPM()
		self.progressMeterInit("Reading lockbits", 0)
		self.__sendInstr(SDI=0x04, SII=0x4C)
		self.__sendInstr(SDI=0x00, SII=0x78)
		self.__sendInstr(SDI=0x00, SII=0x7C)
		self.__readSDOBufferHigh()
		lockbits = int2byte(self.top.cmdReadBufferReg8() | 0xFC)
		self.progressMeterFinish()
		return lockbits

	def writeLockbits(self, image):
		if len(image) != 1:
			self.throwError("Invalid Lockbits image size %d (expected %d)" %\
				(len(image), 1))
		self.__enterPM()
		self.progressMeterInit("Writing lockbits", 0)
		self.__sendInstr(SDI=0x20, SII=0x4C)
		self.__sendInstr(SDI=(byte2int(image[0]) & 3), SII=0x2C)
		self.__sendInstr(SDI=0x00, SII=0x64)
		self.__sendInstr(SDI=0x00, SII=0x6C)
		self.__waitHighSDO()
		self.progressMeterFinish()

	def __readSignature(self):
		self.__sendInstr(SDI=0x08, SII=0x4C)
		for i in range(0, 3):
			self.__sendInstr(SDI=i, SII=0x0C)
			self.__sendInstr(SDI=0x00, SII=0x68)
			self.__sendInstr(SDI=0x00, SII=0x6C)
			self.__readSDOBufferHigh()
		return self.top.cmdReadBufferReg()[0:3]

	def __enterPM(self):
		"Enter HV programming mode."
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(False)
		self.top.cmdSetVCCVoltage(5)
		self.top.cmdSetVPPVoltage(0)
		self.top.cmdSetVPPVoltage(12)
		self.applyGND(True)
		self.applyVCC(True)

		self.__setPins(SCI=0, SDO_en=0, RST_en=1, RST=0)
		for i in range(0, 6):
			self.__setPins(SCI=0, SDO_en=0, RST_en=1, RST=0)
			self.__setPins(SCI=1, SDO_en=0, RST_en=1, RST=0)
		self.__setPins(SCI=0, SDO_en=1, SDO=0, RST_en=1, RST=0)
		self.top.hostDelay(0.001)
		self.__setPins(SDO_en=1, SDO=0, RST_en=0)
		self.applyVPP(True)
		self.top.hostDelay(0.001)
		self.__setPins(SDO_en=0)
		self.top.hostDelay(0.01)

		signature = self.__readSignature()
		if signature != self.signature:
			msg = "Unexpected device signature. " +\
			      "Want %02X%02X%02X, but got %02X%02X%02X" % \
				(byte2int(self.signature[0]), byte2int(self.signature[1]),
				 byte2int(self.signature[2]),
				 byte2int(signature[0]), byte2int(signature[1]),
				 byte2int(signature[2]))
			if self.top.getForceLevel() >= 1:
				self.printWarning(msg)
			else:
				self.throwError(msg)

	def __sendReadEEPROMInstr(self):
		self.__sendInstr(SDI=0x03, SII=0x4C)

	def __sendWriteEEPROMInstr(self):
		self.__sendInstr(SDI=0x11, SII=0x4C)

	def __sendReadFlashInstr(self):
		self.__sendInstr(SDI=0x02, SII=0x4C)

	def __sendWriteFlashInstr(self):
		self.__sendInstr(SDI=0x10, SII=0x4C)

	def __sendNOP(self):
		self.__sendInstr(SDI=0x00, SII=0x4C)

	def __sendInstr(self, SDI, SII):
		self.__setSDI(SDI)
		self.__setSII(SII)
		self.__loadCommand(self.PROGCMD_SENDINSTR)
		# We do not poll the busy flag, because that would result
		# in a significant slowdown. We delay long enough for the
		# command to finish execution, instead.
		self.top.hostDelay(0.001)

	def __setSDI(self, sdi):
		self.top.cmdFPGAWrite(0x13, sdi & 0xFF)

	def __setSII(self, sii):
		self.top.cmdFPGAWrite(0x14, sii & 0xFF)

	def __loadCommand(self, command):
		self.top.cmdFPGAWrite(0x12, command & 0xFF)

	def __runCommandSync(self, command):
		self.__loadCommand(command)
		self.__busyWait()

	def __setPins(self, SCI=0, SDO_en=0, SDO=0, RST_en=0, RST=0):
		data = 0
		if SCI:
			data |= 1
		if SDO_en:
			data |= 2
		if SDO:
			data |= 4
		if RST_en:
			data |= 8
		if RST:
			data |= 16
		self.top.cmdFPGAWrite(0x15, data)

	def __getStatusFlags(self):
		self.top.cmdFPGARead(0x12)
		stat = self.top.cmdReadBufferReg()
		return byte2int(stat[0])

	def __readSDOBufferHigh(self):
		self.top.cmdFPGARead(0x10)

	def __rawSDOState(self):
		return bool(self.__getStatusFlags() & self.STAT_SDO)

	def __busy(self):
		return bool(self.__getStatusFlags() & self.STAT_BUSY)

	def __busyWait(self):
		for i in range(0, 100):
			if not self.__busy():
				return
			self.top.hostDelay(0.01)
		self.throwError("Timeout in busywait.")

	def __waitHighSDO(self):
		for i in range(0, 100):
			if self.__rawSDOState():
				return
			self.top.hostDelay(0.01)
		self.throwError("Timeout waiting for SDO.")

ChipDescription(
	Chip_AtTiny13dip8,
	bitfile = "attiny13dip8",
	runtimeID = (0x0001, 0x01),
	chipVendors = "Atmel",
	description = "AtTiny13",
	packages = ( ("DIP8", ""), ),
)



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC18F2320 DIP18
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_18f2221family import *

class Chip_PIC18F2321dip28(microchip8_18f2221family):

	hasEEPROM = True

     	writeBufferSize			 = 8
     	eraseBufferSize			 = 64
     	
     	def __init__(self):
     		microchip8_18f2221family.__init__(self,
			chipPackage="DIP28",
			chipPinVCC=20,
			chipPinsVPP=1,
			chipPinGND=19,
			signature="\x22\x21",
			flashPageSize=0x2000,
			flashPages=1,
			eepromPageSize=0x100,
			eepromPages=1,
			fuseBytes=14
			)
			

fuseDesc = (
	BitDescription(0o00, "NA"),
	BitDescription(0o01, "NA"),
	BitDescription(0o02, "NA"),
	BitDescription(0o03, "NA"),
	BitDescription(0o04, "NA"),
	BitDescription(0o05, "NA"),
	BitDescription(0o06, "NA"),
	BitDescription(0o07, "NA"),
	BitDescription(0o10, "FOSC[0], 0000=LP, 1000=internal RC oscillator, RA6=CLKO"),
	BitDescription(0o11, "FOSC[1]"),
	BitDescription(0o12, "FOSC[2]"),
	BitDescription(0o13, "FOSC[3]"),
	BitDescription(0o14, "NA"),
	BitDescription(0o15, "NA"),
	BitDescription(0o16, "FCMEN, 0=Fail-Safe Clock Monitor is disabled"),
	BitDescription(0o17, "IESO, 0=Internal/External Switchover mode is disabled"),
    
	BitDescription(0o20, "nPWRTEN"),
	BitDescription(0o21, "BOREN[0]"),
	BitDescription(0o22, "BOREN[1]"),
	BitDescription(0o23, "BORV[0]"),
	BitDescription(0o24, "BORV[1]"),
	BitDescription(0o25, "NA"),
	BitDescription(0o26, "NA"),
	BitDescription(0o27, "NA"),
	BitDescription(0o30, "WDTEN, 0=WDT disabled, 1=WDT enabled"),
	BitDescription(0o31, "WDTPS[0]"),
	BitDescription(0o32, "WDTPS[1]"),
	BitDescription(0o33, "WDTPS[2]"),
	BitDescription(0o34, "WDTPS[3]"),
	BitDescription(0o35, "NA"),
	BitDescription(0o36, "NA"),
	BitDescription(0o37, "NA"),
	
	BitDescription(0o40, "NA"),
	BitDescription(0o41, "NA"),
	BitDescription(0o42, "NA"),
	BitDescription(0o43, "NA"),
	BitDescription(0o44, "NA"),
	BitDescription(0o45, "NA"),
	BitDescription(0o46, "NA"),
	BitDescription(0o47, "NA"),
	BitDescription(0o50, "NA"),
	BitDescription(0o51, "PBADEN"),
	BitDescription(0o52, "LPT1OSC"),
	BitDescription(0o53, "NA"),
	BitDescription(0o54, "NA"),
	BitDescription(0o55, "NA"),
	BitDescription(0o56, "NA"),
	BitDescription(0o57, "MCLRE"), 	
    
	BitDescription(0o60, "STVREN"),
	BitDescription(0o61, "NA"),
	BitDescription(0o62, "LVP"),
	BitDescription(0o63, "NA"),
	BitDescription(0o64, "BBSIZ[0]"),
	BitDescription(0o65, "BBSIZ[1]"),
	BitDescription(0o66, "XINST"),
	BitDescription(0o67, "nDEBUG"),
	BitDescription(0o70, "NA"),
	BitDescription(0o71, "NA"),
	BitDescription(0o72, "NA"),
	BitDescription(0o73, "NA"),
	BitDescription(0o74, "NA"),
	BitDescription(0o75, "NA"),
	BitDescription(0o76, "NA"),
	BitDescription(0o77, "NA"),
	
	BitDescription(0o100, "CP[0]"),
	BitDescription(0o101, "CP[1]"),
	BitDescription(0o102, "CP[2]/NA"),
	BitDescription(0o103, "CP[3]/NA"),
	BitDescription(0o104, "CP[4]/NA"),
	BitDescription(0o105, "CP[5]/NA"),
	BitDescription(0o106, "NA"),
	BitDescription(0o107, "NA"),
	BitDescription(0o110, "NA"),
	BitDescription(0o111, "NA"),
	BitDescription(0o112, "NA"),
	BitDescription(0o113, "NA"),
	BitDescription(0o114, "NA"),
	BitDescription(0o115, "NA"),
	BitDescription(0o116, "CPB"),
	BitDescription(0o117, "CPD"), 	
	
	BitDescription(0o120, "WRT[0]"),
	BitDescription(0o121, "WRT[1]"),
	BitDescription(0o122, "WRT[2]/NA"),
	BitDescription(0o123, "WRT[3]/NA"),
	BitDescription(0o124, "WRT[4]/NA"),
	BitDescription(0o125, "WRT[5]/NA"),
	BitDescription(0o126, "NA"),
	BitDescription(0o127, "NA"),
	BitDescription(0o130, "NA"),
	BitDescription(0o131, "NA"),
	BitDescription(0o132, "NA"),
	BitDescription(0o133, "NA"),
	BitDescription(0o134, "NA"),
	BitDescription(0o135, "WRTC"),
	BitDescription(0o136, "WRTB"),
	BitDescription(0o137, "WRTD"),
	
	BitDescription(0o140, "EBTR[0]"),
	BitDescription(0o141, "EBTR[1]"),
	BitDescription(0o142, "EBTR[2]/NA"),
	BitDescription(0o143, "EBTR[3]/NA"),
	BitDescription(0o144, "EBTR[4]/NA"),
	BitDescription(0o145, "EBTR[5]/NA"),
	BitDescription(0o146, "NA"),
	BitDescription(0o147, "NA"),
	BitDescription(0o150, "NA"),
	BitDescription(0o151, "NA"),
	BitDescription(0o152, "NA"),
	BitDescription(0o153, "NA"),
	BitDescription(0o154, "NA"),
	BitDescription(0o155, "NA"),
	BitDescription(0o156, "EBTRB"),
	BitDescription(0o157, "NA"),
)

ChipDescription(
	Chip_PIC18F2321dip28,
	bitfile="microchip01dip28",
	chipID="PIC18F2321dip28",
	runtimeID=(0xDE07, 0x01),
	chipVendors="Microchip",
	description="PIC18F2321",
	packages=(("DIP28", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#    27c16dip28  UV/OTP EPROM
#    27c32dip28  UV/OTP EPROM
#    27c64dip28  UV/OTP EPROM
#    27c128dip28 UV/OTP EPROM
#    27c256dip28 UV/OTP EPROM
#    27c512dip28 UV/OTP EPROM
#    Various manufacturers
#
#    Copyright (c) 2012 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *
from libtoprammer.util import *


class Chip_27cXXX(Chip):
	"Generic 27cXXX EPROM"

	CTYPE_16	= 0
	CTYPE_32	= 1
	CTYPE_64	= 2
	CTYPE_128	= 3
	CTYPE_256	= 4
	CTYPE_512	= 5

	# Chip sizes (in bytes)
	ctype2size = {
		CTYPE_16	: 16 * 1024 // 8,
		CTYPE_32	: 32 * 1024 // 8,
		CTYPE_64	: 64 * 1024 // 8,
		CTYPE_128	: 128 * 1024 // 8,
		CTYPE_256	: 256 * 1024 // 8,
		CTYPE_512	: 512 * 1024 // 8,
	}

	# Programming voltages
	vppTable = {
		CTYPE_16	: 12.75,
		CTYPE_32	: 12.75,
		CTYPE_64	: 12.75,
		CTYPE_128	: 12.75,
		CTYPE_256	: 12.75,
		CTYPE_512	: 12.75,
	}

	# Programming pulse lengths (in microseconds)
	ppulseLengths = {
		CTYPE_16	: 1000,
		CTYPE_32	: 1000,
		CTYPE_64	: 1000,
		CTYPE_128	: 1000,
		CTYPE_256	: 500,
		CTYPE_512	: 500,
	}

	# Can we read the chip with VPP enabled?
	readWithVPP = {
		CTYPE_16	: True,
		CTYPE_32	: False, # VPP is shared with OE
		CTYPE_64	: True,
		CTYPE_128	: True,
		CTYPE_256	: True,
		CTYPE_512	: False, # VPP is shared with OE
	}

	# Chips that need overprogramming pulse.
	# This may not be true for all manufacturers.
	# Let the user set the chip option 'overprogram_pulse' in that case.
	needOverprogram = {
		CTYPE_16	: True,
		CTYPE_32	: True,
		CTYPE_64	: True,
		CTYPE_128	: True,
		CTYPE_256	: True,
		CTYPE_512	: False,
	}

	def __init__(self, chipType,
		     chipPinVCC, chipPinVPP, chipPinGND):
		Chip.__init__(self, chipPackage = "DIP28",
			      chipPinVCC = chipPinVCC,
			      chipPinsVPP = chipPinVPP,
			      chipPinGND = chipPinGND)
		self.chipType = chipType
		self.generic = GenericAlgorithms(self)
		self.addrSetter = AddrSetter(self, 0, 1)

	def readEEPROM(self):
		self.__turnOn()
		return self.generic.simpleReadEPROM(
			sizeBytes = self.ctype2size[self.chipType],
			readData8Func = self.__dataRead,
			addrSetter = self.addrSetter,
			initFunc = lambda: self.__setFlags(oe=0, ce=0),
			exitFunc = lambda: self.__setFlags(oe=1, ce=1)
		)

	def writeEEPROM(self, image):
		sizeBytes = self.ctype2size[self.chipType]
		if len(image) > sizeBytes:
			self.throwError("Invalid image size. "
				"Got %d bytes, but EPROM is only %d bytes." %\
				(len(image), sizeBytes))

		# Get the options
		immediateVerify = self.getChipOptionValue(
			"immediate_verify",
			self.readWithVPP[self.chipType])
		overprogram = self.getChipOptionValue(
			"overprogram_pulse",
			self.needOverprogram[self.chipType])
		vppVolt = self.getChipOptionValue(
			"vpp_voltage",
			self.vppTable[self.chipType])
		progpulseUsec = self.getChipOptionValue(
			"ppulse_length",
			self.ppulseLengths[self.chipType])

		# Run the write algorithm
		self.__writeAlgo(image, vppVolt, immediateVerify, overprogram,
				 progpulseUsec)

	def __writeAlgo(self, image, vppVolt, immediateVerify, overprogramPulse,
			progpulseUsec):
		self.printInfo("Using %.2f VPP" % vppVolt)
		self.printInfo("Using %s verify." %\
			("immediate" if immediateVerify else "detached"))
		if immediateVerify and not self.readWithVPP[self.chipType]:
			self.printWarning("Immediate verify will be slow "
				"on this chip!")
		self.printInfo("%s overprogramming pulse." %\
			("Using" if overprogramPulse else "Not using"))
		if not immediateVerify and overprogramPulse:
			self.printWarning("Using overprogramming, but no "
				"immediate verify. This probably is NOT what "
				"you intended.")
		self.printInfo("Using ppulse length: %d microseconds" %\
			progpulseUsec)

		self.__turnOn()
		self.addrSetter.reset()
		self.applyVPP(False)
		self.top.cmdSetVPPVoltage(vppVolt)
		okMask = [ False ] * len(image)
		nrRetries = 25
		for retry in range(0, nrRetries):
			# Program
			self.progressMeterInit("Writing EPROM", len(image))
			self.__setFlags(data_en=1, prog_en=1, ce=0, oe=1)
			self.applyVPP(True)
			for addr in range(0, len(image)):
				self.progressMeter(addr)
				if okMask[addr]:
					continue
				data = byte2int(image[addr])
				if data == 0xFF:
					okMask[addr] = True
				else:
					self.__writeByte(addr, data,
						immediateVerify, overprogramPulse,
						progpulseUsec)
			self.applyVPP(False)
			self.__setFlags(data_en=0, prog_en=0, ce=0, oe=0)
			self.progressMeterFinish()
			if immediateVerify:
				break
			if all(okMask):
				break
			# Detached verify
			readImage = self.generic.simpleReadEPROM(
				sizeBytes = len(image),
				readData8Func = self.__dataRead,
				addrSetter = self.addrSetter,
				initFunc = lambda: self.__setFlags(oe=0, ce=0),
				exitFunc = lambda: self.__setFlags(oe=1, ce=1)
			)
			for addr in range(0, len(image)):
				if okMask[addr]:
					continue
				if image[addr] == readImage[addr]:
					okMask[addr] = True
			if all(okMask):
				break
			self.printInfo("%d of %d bytes failed verification. "
				"Retrying those bytes..." %\
				(len([ ok for ok in okMask if ok]),
				 len(okMask)))
		else:
			self.throwError("Failed to write EPROM. "
				"Tried %d times." % nrRetries)
		self.__setFlags()
		self.top.cmdSetVPPVoltage(5)

	def __writeByte(self, addr, data,
			doVerify, doOverprogram, progpulseUsec):
		self.addrSetter.load(addr)
		self.__setDataPins(data)
		for retry in range(0, 25):
			self.__progPulse(progpulseUsec)
			if not doVerify:
				break
			# Immediate verify
			if not self.readWithVPP[self.chipType]:
				self.applyVPP(False)
			self.__setFlags(data_en=0, prog_en=0, ce=0, oe=0)
			readData = self.__readByte(addr)
			self.__setFlags(data_en=1, prog_en=1, ce=0, oe=1)
			if not self.readWithVPP[self.chipType]:
				self.applyVPP(True)
			if readData == data:
				break
		else:
			self.throwError("Failed to write EPROM address %d" % addr)
		if doOverprogram:
			self.addrSetter.load(addr)
			self.__progPulse(progpulseUsec * 3 * (retry + 1))

	def __readByte(self, addr):
		self.addrSetter.load(addr)
		self.__dataRead()
		return self.top.cmdReadBufferReg8()

	def __turnOn(self):
		self.__setType(self.chipType)
		self.__setFlags()
		self.generic.simpleVoltageSetup()

	def __setDataPins(self, value):
		self.top.cmdFPGAWrite(2, value)

	def __setFlags(self, data_en=0, prog_en=0, ce=1, oe=1):
		value = 0
		if data_en:
			value |= (1 << 0)
		if prog_en:
			value |= (1 << 1)
		if ce:
			value |= (1 << 2)
		if oe:
			value |= (1 << 3)
		self.top.cmdFPGAWrite(3, value)

	def __progPulse(self, usec):
		value = roundup(usec, 100) // 100
		if value > 0x7F:
			value = roundup(value, 8) // 8
			if value > 0x7F:
				self.throwError("__progPulse time too big")
			value |= 0x80 # Set "times 8" multiplier
		self.top.cmdFPGAWrite(4, value)
		seconds = float(value & 0x7F) / 10000
		if value & 0x80:
			seconds *= 8
		self.top.cmdDelay(seconds)
		self.top.flushCommands()

	def __setType(self, typeNumber):
		self.top.cmdFPGAWrite(5, typeNumber)

	def __dataRead(self):
		self.top.cmdFPGARead(0)

class ChipDescription_27cXXX(ChipDescription):
	"Generic 27cXXX ChipDescription"

	def __init__(self, chipImplClass, name):
		ChipDescription.__init__(self,
			chipImplClass = chipImplClass,
			bitfile = "_27cxxxdip28",
			chipID = name,
			runtimeID = (12, 1),
			chipType = ChipDescription.TYPE_EPROM,
			chipVendors = "Various",
			description = name + " EPROM",
			packages = ( ("DIP28", ""), ),
			chipOptions = (
				ChipOptionBool("immediate_verify",
					"Immediately verify each written byte"),
				ChipOptionBool("overprogram_pulse",
					"Perform an 'overprogramming' pulse"),
				ChipOptionFloat("vpp_voltage",
					"Override the default VPP voltage",
					minVal=10.0, maxVal=14.0),
				ChipOptionInt("ppulse_length",
					"Force 'Programming pulse' length, in microseconds.",
					minVal=100, maxVal=10000),
			)
		)

class Chip_27c16(Chip_27cXXX):
	def __init__(self):
		Chip_27cXXX.__init__(self,
			chipType = Chip_27cXXX.CTYPE_16,
			chipPinVCC = 26,
			chipPinVPP = 23,
			chipPinGND = 14)

ChipDescription_27cXXX(Chip_27c16, "27c16")

class Chip_27c32(Chip_27cXXX):
	def __init__(self):
		Chip_27cXXX.__init__(self,
			chipType = Chip_27cXXX.CTYPE_32,
			chipPinVCC = 26,
			chipPinVPP = 22,
			chipPinGND = 14)

ChipDescription_27cXXX(Chip_27c32, "27c32")

class Chip_27c64(Chip_27cXXX):
	def __init__(self):
		Chip_27cXXX.__init__(self,
			chipType = Chip_27cXXX.CTYPE_64,
			chipPinVCC = 28,
			chipPinVPP = 1,
			chipPinGND = 14)

ChipDescription_27cXXX(Chip_27c64, "27c64")

class Chip_27c128(Chip_27cXXX):
	def __init__(self):
		Chip_27cXXX.__init__(self,
			chipType = Chip_27cXXX.CTYPE_128,
			chipPinVCC = 28,
			chipPinVPP = 1,
			chipPinGND = 14)

ChipDescription_27cXXX(Chip_27c128, "27c128")

class Chip_27c256(Chip_27cXXX):
	def __init__(self):
		Chip_27cXXX.__init__(self,
			chipType = Chip_27cXXX.CTYPE_256,
			chipPinVCC = 28,
			chipPinVPP = 1,
			chipPinGND = 14)

ChipDescription_27cXXX(Chip_27c256, "27c256")

class Chip_27c512(Chip_27cXXX):
	def __init__(self):
		Chip_27cXXX.__init__(self,
			chipType = Chip_27cXXX.CTYPE_512,
			chipPinVCC = 28,
			chipPinVPP = 22,
			chipPinGND = 14)

ChipDescription_27cXXX(Chip_27c512, "27c512")



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC24f08kl402 DIP28
#
#    Copyright (c) 2014 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip16_common import *
from configWords import klx0x_fuseDesc

class Chip_Pic24f08kl402dip28(Chip_Microchip16_common):
	
	voltageVDD = 3.3
	voltageVPP = 8
	
	logicalFlashProgramMemorySize = 0x800000
	logicalFlashConfigurationMemorySize = 0x800000
	
	def __init__(self):
	 	Chip_Microchip16_common.__init__(self,
		chipPackage="DIP28",
		chipPinVCC=28,
		chipPinsVPP=1,
		chipPinGND=27,
		signature="\x04\x4b",
		# flashPageSize (in number of 24bit words)
		flashPageSize=0x15fe / 2 + 2,
		# flashPageSize=0x40,
		flashPages=1,
		# eepromPageSize (in 16bit words)
		eepromPageSize=0x100,
		eepromPages=1,
		# all 7 words uses lowest byte only
		fuseBytes=2 * 9
		)
		self.configWordAddr = 0xF80000
		# self.osccalBackupAddr = self.userIDLocationAddr + self.userIDLocationSize

fuseDesc = klx0x_fuseDesc

ChipDescription(
	Chip_Pic24f08kl402dip28,
	bitfile="microchip16dip28",
	chipID="pic24f08kl402dip28",
	runtimeID=(0xDF02, 0x01),
	chipVendors="Microchip",
	description="PIC24F08KL402",
	packages=(("DIP28", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#    74HC4094 unit-tester
#
#    Copyright (c) 2011 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from unitest import *

class Chip_74hc4094(Chip_Unitest):
	def __init__(self):
		Chip_Unitest.__init__(self, chipPackage="DIP16",
					    chipPinVCC=16,
					    chipPinGND=8,
					    VCCVoltage=5)

	def __initChip(self):
		self.zifPin_STR = self.generator.getZifPinForPackagePin(1)
		self.zifPin_D = self.generator.getZifPinForPackagePin(2)
		self.zifPin_CP = self.generator.getZifPinForPackagePin(3)
		self.zifPin_OE = self.generator.getZifPinForPackagePin(15)
		self.zifPin_QS1 = self.generator.getZifPinForPackagePin(9)
		self.zifPin_QS2 = self.generator.getZifPinForPackagePin(10)
		self.zifPins_QP = []
		for packagePin in (4, 5, 6, 7, 14, 13, 12, 11):
			self.zifPins_QP.append(
				self.generator.getZifPinForPackagePin(packagePin))

		self.reset()

		outen = 0
		for zifPin in (self.zifPin_STR, self.zifPin_D,
			       self.zifPin_CP, self.zifPin_OE):
			outen |= (1 << (zifPin - 1))
		self.setOutputEnableMask(outen)

		self.applyGND(True)
		self.applyVCC(True)

	def test(self):
		testPatterns = (0xFF, 0x00, 0xAA, 0x55, 0xF0, 0x0F,
				0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80,
				0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01, 0x00)

		self.progressMeterInit("Logic test", len(testPatterns))
		self.__initChip()

		# Initialize the register to all-zero
		self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=0, OE=0))
		for i in range(0, 9):
			self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=1, OE=0))
			self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=0, OE=0))
		self.setOutputs(self.__makeOutMask(STR=1, D=0, CP=0, OE=0))
		self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=0, OE=1))

		QP = self.__readQP()
		if QP != 0x00:
			self.throwError("Failed to clear the shiftregister. Got 0x%02X" % QP)

		prevContents = 0x00
		prev_QS1 = False
		count = 0
		for testPattern in testPatterns:
			self.progressMeter(count)
			self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=0, OE=0))
			for bitNr in range(7, -1, -1): # MSB first
				self.setOutputs(self.__makeOutMask(STR=0,
								   D=(testPattern & (1 << bitNr)),
								   CP=0, OE=0))
				self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=1, OE=0))
				(QS1, QS2) = self.__readQS()
				word = (prevContents << 8) | testPattern
				expect_QS1 = bool(word & (1 << (bitNr + 7)))
				expect_QS2 = prev_QS1
				prev_QS1 = expect_QS1
				if QS1 != expect_QS1 or QS2 != expect_QS2:
					self.throwError("Got invalid QS serial output for "
						"test pattern 0x%02X bit %d. Got %d/%d, but "
						"expected %d/%d" %\
						(testPattern, bitNr, QS1, QS2,
						 expect_QS1, expect_QS2))
				self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=0, OE=0))
			self.setOutputs(self.__makeOutMask(STR=1, D=0, CP=0, OE=0))
			self.setOutputs(self.__makeOutMask(STR=0, D=0, CP=0, OE=1))

			QP = self.__readQP()
			if QP != testPattern:
				self.throwError("Failed on test pattern 0x%02X. Got 0x%02X" %\
						(testPattern, QP))

			prevContents = testPattern
			count += 1
		self.progressMeterFinish()

	def __makeOutMask(self, STR, D, CP, OE):
		mask = 0
		if STR:
			mask |= (1 << (self.zifPin_STR - 1))
		if D:
			mask |= (1 << (self.zifPin_D - 1))
		if CP:
			mask |= (1 << (self.zifPin_CP - 1))
		if OE:
			mask |= (1 << (self.zifPin_OE - 1))
		return mask

	def __readQP(self):
		QPValue = 0
		count = 0
		inputs = self.getInputs()
		for zifPin in self.zifPins_QP:
			if inputs & (1 << (zifPin - 1)):
				QPValue |= (1 << count)
			count += 1
		return QPValue

	def __readQS(self):
		QS1 = False
		QS2 = False
		inputs = self.getInputs()
		if inputs & (1 << (self.zifPin_QS1 - 1)):
			QS1 = True
		if inputs & (1 << (self.zifPin_QS2 - 1)):
			QS2 = True
		return (QS1, QS2)

ChipDescription(
	Chip_74hc4094,
	chipID = "74hc4094dip16",
	bitfile = "unitest",
	runtimeID = (0x0008, 0x01),
	chipType = ChipDescription.TYPE_LOGIC,
	description = "74HC(T)4094 shift-register",
	chipVendors = ("Philips", "Other"),
)
